/*
 * nScore
 *
 * game.c
 *
 * Copyright (C) 1996-2003 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE

#include "General.h"
#include "Message.h"
#include "game.h"
#include "board.h"
#include "score.h"
#include "score_input.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("game_wnd")

// XPe[}ύXʒm
#ifndef WM_THEMECHANGED
#define WM_THEMECHANGED				0x031A
#endif

/* Global Variables */
static HWND board_wnd;
static HWND score_wnd;

extern HINSTANCE hInst;
extern COLOR_INFO ci;

/* Local Function Prototypes */
static RECT get_window_size(const HWND hWnd);
static BOOL set_window_size(const HWND hWnd);
static LRESULT CALLBACK game_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_window_size - EBhE̍ŏTCY擾
 */
static RECT get_window_size(const HWND hWnd)
{
	RECT window_rect, client_rect;
	RECT borad_rect, score_rect;
	RECT ret;
	int height;

	GetClientRect(hWnd, &client_rect);
	GetWindowRect(hWnd, &window_rect);
	GetWindowRect(board_wnd, &borad_rect);
	GetWindowRect(score_wnd, &score_rect);

	height = (borad_rect.bottom - borad_rect.top > score_rect.bottom - score_rect.top) ?
		borad_rect.bottom - borad_rect.top : score_rect.bottom - score_rect.top;

	ret.left = ret.top = 0;
	ret.right = (borad_rect.right - borad_rect.left) + (score_rect.right - score_rect.left) +
		(window_rect.right - window_rect.left - client_rect.right);
	ret.bottom = height + (window_rect.bottom - window_rect.top - client_rect.bottom);
	return ret;
}

/*
 * set_window_size - EBhETCY̐ݒ
 */
static BOOL set_window_size(const HWND hWnd)
{
	RECT client_rect;
	RECT borad_rect, score_rect;
	int width, height;
	int board_left, board_width;
	int score_left, score_width;

	GetClientRect(hWnd, &client_rect);
	GetWindowRect(board_wnd, &borad_rect);
	GetWindowRect(score_wnd, &score_rect);

	// Q[EBhẼTCYݒ
	width = client_rect.right - client_rect.left;
	height = client_rect.bottom - client_rect.top;
	board_width = borad_rect.right - borad_rect.left;
	score_width = score_rect.right - score_rect.left;

	// {[ḧʒuݒ
	board_left = (int)(width * ((float)board_width / (float)(board_width + score_width)) - board_width) / 2;
	if (board_left < 0) {
		board_left = 0;
	}
	SetWindowPos(board_wnd, 0,
		board_left, (height - (borad_rect.bottom - borad_rect.top)) / 2,
		0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER);

	// XRA\̈ʒuݒ
	score_left = (int)(width * ((float)board_width / (float)(board_width + score_width))) +
		(int)(width * ((float)score_width / (float)(board_width + score_width)) - score_width) / 2;
	if (score_left + score_width > width) {
		score_left = board_left + board_width;
	}
	SetWindowPos(score_wnd, 0,
		score_left, (height - (score_rect.bottom - score_rect.top)) / 2,
		0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER);
	return TRUE;
}

/*
 * game_proc - Q[EBhEvV[W
 */
static LRESULT CALLBACK game_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	HBRUSH draw_brush;
	BOOL enable_flag;

	switch (msg) {
	case WM_CREATE:
		// {[h̍쐬
		board_wnd = board_create(hInst, hWnd, 0, (int)((CREATESTRUCT *)lParam)->lpCreateParams);
		// XRAEBhE̍쐬
		score_wnd = score_create(hInst, hWnd, 0);
		SendMessage(score_wnd, WM_SET_DRAW_MESSAGE, 0, (LPARAM)message_get_res(IDS_STRING_MSG_WAIT));
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;
	
	case WM_DESTROY:
		// EBhE̔j
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		SetFocus(score_wnd);
		break;

	case WM_SIZE:
		set_window_size(hWnd);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;
	
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		// wi̓hԂ
		draw_brush = CreateSolidBrush(ci.background);
		FillRect(hdc, &ps.rcPaint, draw_brush);
		DeleteObject(draw_brush);
		EndPaint(hWnd, &ps);
		break;

	case WM_ERASEBKGND:
		return 1;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case ID_ACCEL_RETURN:
		case IDC_BUTTON_NEXT:
			SendMessage(score_wnd, WM_COMMAND, wParam, lParam);
			break;
		}
		break;

	case WM_GET_WINDOW_SIZE:
		// EBhE̍ŏTCY擾
		*((RECT *)lParam) = get_window_size(hWnd);
		break;

	case WM_SET_BOARD_SIZE:
		// {[hTCYύX
		enable_flag = IsWindowEnabled(board_wnd);
		SendMessage(board_wnd, WM_CLOSE, 0, 0);
		board_wnd = board_create(hInst, hWnd, 0, lParam);
		SendMessage(board_wnd, WM_INIT_SELECT, 0, 0);
		SendMessage(board_wnd, WM_SET_BOARD_ENABLE, enable_flag, 0);
		break;

	case WM_GAME_END:
	case WM_SET_STATUS:
		return SendMessage(GetParent(hWnd), msg, wParam, lParam);

	case WM_INIT_SELECT:
	case WM_SET_BOARD_ENABLE:
	case WM_SET_BULL_TYPE:
		return SendMessage(board_wnd, msg, wParam, lParam);

	case WM_SET_DRAW_MESSAGE:
	case WM_GAME_INIT:
	case WM_GAME_STOP:
	case WM_SHOW_SCORE:
	case WM_SAVE_SCORE:
	case WM_PREV_PLAYER:
	case WM_SET_SCORE:
	case WM_SET_INPUT_ENABLE:
		return SendMessage(score_wnd, msg, wParam, lParam);

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * game_regist - EBhENX̓o^
 */
BOOL game_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)game_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * game_create - EBhE̍쐬
 */
HWND game_create(const HINSTANCE hInstance, const HWND pWnd, int id, int size)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindowEx(WS_EX_CLIENTEDGE,
		WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_VISIBLE | WS_TABSTOP,
		0, 0, 100, 100, pWnd, (HMENU)id, hInstance, (LPVOID)size);
	return hWnd;
}
/* End of source */
