/*
 * nScore
 *
 * main.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <shlobj.h>
#include <commctrl.h>

#include "resource.h"

#include "General.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "StatusBar.h"
#include "shortcut.h"
#include "ini.h"
#include "game.h"
#include "board.h"
#include "score.h"
#include "score_01.h"
#include "score_cricket.h"
#include "score_countup.h"
#include "score_cricket_countup.h"
#include "score_eagleseye.h"
#include "score_halfit.h"
#include "score_rotation.h"
#include "score_splat.h"
#include "score_input.h"

/* Define */
#define MAIN_WND_CLASS				TEXT("nscore_wnd")
#define WINDOW_TITLE				TEXT("nScore")

#define INI_FILE					TEXT("nScore.ini")
#define LNK_FILE					TEXT("nScore.lnk")

#define ID_TIMER_INIT				1
#define TIMER_INTERVAL_INIT			1

#define BOARD_SIZE_L				135
#define BOARD_SIZE_M				100
#define BOARD_SIZE_S				75

/* Global Variables */
static HWND game_wnd;
static HWND status_wnd;
static int window_width;
static int window_height;
static int init_game = 0;

HINSTANCE hInst;
OPTION_INFO op;
COLOR_INFO ci;
BYTE char_set;

TCHAR ini_path[MAX_PATH];

/* Local Function Prototypes */
static void set_cursor(const BOOL wiat_flag);
static void set_enable_menu(const HWND hWnd, const BOOL game_flag);
static BOOL init_window_size(const HWND hWnd);
static BOOL set_window_size(const HWND hWnd);
static LRESULT CALLBACK main_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);
static void commnad_line_func(void);

/*
 * set_cursor - J[\ݒ
 */
static void set_cursor(const BOOL wiat_flag)
{
	static HCURSOR old_cursor;

	if (wiat_flag == TRUE) {
		// vJ[\ɂ
		old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	} else {
		if (old_cursor == NULL) {
			SetCursor(LoadCursor(NULL, IDC_ARROW));
		} else {
			// ̃J[\ɖ߂
			SetCursor(old_cursor);
			old_cursor = NULL;
		}
	}
}

/*
 * ser_enable_menu - j[Ԃ̐ݒ
 */
static void set_enable_menu(const HWND hWnd, const BOOL game_flag)
{
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_01, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_CRICKET, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_COUNTUP, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_CRICKET_COUNTUP, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_EAGLESEYE, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_HALFIT, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_ROTATION, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SPLAT, (game_flag == TRUE) ? MF_GRAYED : MF_ENABLED);

	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_PREV, (game_flag == TRUE) ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_GAME_END, (game_flag == TRUE) ? MF_ENABLED : MF_GRAYED);
}

/*
 * init_window_size - EBhETCY̏
 */
static BOOL init_window_size(const HWND hWnd)
{
	RECT window_rect, client_rect;
	RECT game_rect, status_rect;

	GetClientRect(hWnd, &client_rect);
	GetWindowRect(hWnd, &window_rect);
	GetWindowRect(status_wnd, &status_rect);
	SendMessage(game_wnd, WM_GET_WINDOW_SIZE, 0, (LPARAM)&game_rect);

	// EBhE̍ŏTCY
	window_width = (game_rect.right - game_rect.left) + (window_rect.right - window_rect.left - client_rect.right);
	window_height = (game_rect.bottom - game_rect.top) + (window_rect.bottom - window_rect.top - client_rect.bottom);
	if (GetWindowLong(status_wnd, GWL_STYLE) & WS_VISIBLE) {
		window_height += (status_rect.bottom - status_rect.top);
	}
	return TRUE;
}

/*
 * set_window_size - EBhETCY̐ݒ
 */
static BOOL set_window_size(const HWND hWnd)
{
	RECT window_rect, client_rect;
	RECT status_rect;
	int height;

	if (IsIconic(hWnd) != 0) {
		return FALSE;
	}
	if (IsZoomed(hWnd) == 0) {
		// EBhETCY␳
		GetWindowRect(hWnd, &window_rect);
		if (window_rect.right - window_rect.left < window_width) {
			SetWindowPos(hWnd, 0, 0, 0, window_width, window_rect.bottom - window_rect.top, SWP_NOMOVE | SWP_NOZORDER);
		}
		if (window_rect.bottom - window_rect.top < window_height) {
			SetWindowPos(hWnd, 0, 0, 0, window_rect.right - window_rect.left, window_height, SWP_NOMOVE | SWP_NOZORDER);
		}
	}

	GetClientRect(hWnd, &client_rect);
	GetWindowRect(status_wnd, &status_rect);
	// Q[EBhẼTCYݒ
	height = client_rect.bottom - client_rect.top;
	if (GetWindowLong(status_wnd, GWL_STYLE) & WS_VISIBLE) {
		height -= (status_rect.bottom - status_rect.top);
	}
	SetWindowPos(game_wnd, 0, 0, 0,
		client_rect.right - client_rect.left, height, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOZORDER);
	return TRUE;
}

/*
 * main_proc - CEBhEvV[W
 */
static LRESULT CALLBACK main_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	RECT rect;
	TCHAR buf[BUF_SIZE];
	static BOOL game_flag;

	switch (msg) {
	case WM_CREATE:
		commnad_line_func();

		// LN^Zbg̎擾
		{
			HDC hdc;
			hdc = GetDC(hWnd);
			char_set = GetTextCharset(hdc);
			ReleaseDC(hWnd, hdc);
		}

		// Q[EBhE̍쐬
		if (op.board_size == -1) {
			if (GetSystemMetrics(SM_CXSCREEN) < BOARD_SIZE + SCORE_WIDTH) {
				op.board_size = BOARD_SIZE_S;
			} else {
				op.board_size = BOARD_SIZE_M;
			}
		}
		game_wnd = game_create(hInst, hWnd, 0, op.board_size);
		switch (op.board_size) {
		case BOARD_SIZE_L:
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, ID_MENUITEM_BOARD_L, MF_BYCOMMAND);
			break;
		case BOARD_SIZE_M:
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, ID_MENUITEM_BOARD_M, MF_BYCOMMAND);
			break;
		case BOARD_SIZE_S:
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, ID_MENUITEM_BOARD_S, MF_BYCOMMAND);
			break;
		case 0:
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, ID_MENUITEM_BOARD_HIDE, MF_BYCOMMAND);
			break;
		}

		// Xe[^Xo[̍쐬
		status_wnd = statusbar_create(hWnd, 0);
		if (op.window_status_bar == 1) {
			ShowWindow(status_wnd, SW_SHOW);
			CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_STATUS_BAR, MF_CHECKED);
		}

		// EBhẼTCYݒ
		if (op.window_rect.left == 0 && op.window_rect.top == 0 && op.window_rect.right == 0 && op.window_rect.bottom == 0) {
			init_window_size(hWnd);
			// EBhËʒuݒ
			if (IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
				SetWindowPos(hWnd, 0, 0, 0, window_width, window_height, SWP_NOMOVE | SWP_NOZORDER);
			}
			set_window_size(hWnd);
			GetWindowRect(hWnd, &rect);
			SetWindowPos(hWnd, 0,
				(GetSystemMetrics(SM_CXSCREEN) / 2) - ((rect.right - rect.left) / 2),
				(GetSystemMetrics(SM_CYSCREEN) / 2) - ((rect.bottom - rect.top) / 2),
				0, 0, SWP_NOZORDER | SWP_NOSIZE);
		} else {
			init_window_size(hWnd);
			set_window_size(hWnd);
		}
		SendMessage(hWnd, WM_GAME_END, 0, 0);

		EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_GRAYED);
		EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_GRAYED);

		// p^C}[
		SetTimer(hWnd, ID_TIMER_INIT, TIMER_INTERVAL_INIT, NULL);
		break;

	case WM_SIZE:
		op.window_state = (IsZoomed(hWnd) == 0) ? SW_SHOWDEFAULT : SW_MAXIMIZE;
		SendMessage(status_wnd, WM_SIZE, wParam, lParam);
		set_window_size(hWnd);
		break;

	case WM_SIZING:
		if (((LPRECT)lParam)->right - ((LPRECT)lParam)->left < window_width) {
			((LPRECT)lParam)->right = ((LPRECT)lParam)->left + window_width;
		}
		if (((LPRECT)lParam)->bottom - ((LPRECT)lParam)->top < window_height) {
			((LPRECT)lParam)->bottom = ((LPRECT)lParam)->top + window_height;
		}
		return FALSE;

	case WM_EXITSIZEMOVE:
		// TCYύX
		if (IsWindowVisible(hWnd) != 0 && IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
			GetWindowRect(hWnd, (LPRECT)&op.window_rect);
			op.window_rect.right -= op.window_rect.left;
			op.window_rect.bottom -= op.window_rect.top;
		}
		break;

	case WM_QUERYENDSESSION:
		return TRUE;

	case WM_ENDSESSION:
		DestroyWindow(hWnd);
		return 0;

	case WM_CLOSE:
		if (game_flag == TRUE && MessageBox(hWnd,
			message_get_res(IDS_STRING_GAME_END), WINDOW_TITLE,
			MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDNO) {
			break;
		}
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// ݒۑ
		ini_put_option(ini_path);
		PostQuitMessage(0);
		break;

	case WM_SETFOCUS:
		SetFocus(game_wnd);
		break;

	case WM_TIMER:
		switch (wParam) {
		case ID_TIMER_INIT:
			KillTimer(hWnd, ID_TIMER_INIT);
			// {[h̏
			set_cursor(TRUE);
			SendMessage(game_wnd, WM_INIT_SELECT, 0, 0);
			SendMessage(game_wnd, WM_SET_DRAW_MESSAGE, 0, (LPARAM)message_get_res(IDS_STRING_MSG_SCORE));
			set_cursor(FALSE);

			switch (init_game) {
			case GAME_TYPE_01:
				// 01Q[Jn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_01, 0);
				break;
			case GAME_TYPE_CRICKET:
				// NPbgJn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_CRICKET, 0);
				break;
			case GAME_TYPE_COUNTUP:
				// JEgAbvJn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_COUNTUP, 0);
				break;
			case GAME_TYPE_CRICKET_COUNTUP:
				// NPbg JEgAbvJn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_CRICKET_COUNTUP, 0);
				break;
			case GAME_TYPE_EAGLESEYE:
				// C[OY ACJn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_EAGLESEYE, 0);
				break;
			case GAME_TYPE_HALFIT:
				// n[tCbgJn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_HALFIT, 0);
				break;
			case GAME_TYPE_ROTATION:
				// [e[VJn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_ROTATION, 0);
				break;
			case GAME_TYPE_SPLAT:
				// XvbgJn
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_SPLAT, 0);
				break;
			}
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case ID_ACCEL_RETURN:
		case IDC_BUTTON_NEXT:
			SendMessage(game_wnd, WM_COMMAND, wParam, lParam);
			break;

		case ID_MENUITEM_01:
			// 01Q[
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_01, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_01));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_CRICKET:
			// NPbg
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_CRICKET, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_CRICKET));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_COUNTUP:
			// JEgAbv
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_COUNTUP, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_COUNTUP));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_CRICKET_COUNTUP:
			// NPbg JEgAbv
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_CRICKET_COUNTUP, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_CRICKET_COUNTUP));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_EAGLESEYE:
			// C[OY AC
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_EAGLESEYE, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_EAGLESEYE));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_HALFIT:
			// n[tCbg
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_HALFIT, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_HALFIT));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_ROTATION:
			// [e[V
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_ROTATION, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_ROTATION));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_SPLAT:
			// Xvbg
			if (SendMessage(game_wnd, WM_GAME_INIT, GAME_TYPE_SPLAT, 0) == FALSE) {
				break;
			}
			set_enable_menu(hWnd, TRUE);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SCORE_LIST, MF_ENABLED);
			EnableMenuItem(GetMenu(hWnd), ID_MENUITEM_SAVE_SCORE, MF_ENABLED);
			game_flag = TRUE;
			wsprintf(buf, TEXT("%s - %s"), WINDOW_TITLE, message_get_res(IDS_STRING_TITLE_SPLAT));
			SetWindowText(hWnd, buf);
			break;

		case ID_MENUITEM_PREV:
			SendMessage(game_wnd, WM_PREV_PLAYER, 0, 0);
			break;

		case ID_MENUITEM_GAME_END:
			if (MessageBox(hWnd,
				message_get_res(IDS_STRING_GAME_END), WINDOW_TITLE,
				MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDNO) {
				break;
			}
			SetFocus(NULL);
			SendMessage(game_wnd, WM_GAME_STOP, 0, 0);
			SendMessage(hWnd, WM_GAME_END, 0, 0);
			break;

		case ID_MENUITEM_EXIT:
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			break;

		case ID_MENUITEM_BOARD_L:
			// {[hTCYύX ()
			op.board_size = BOARD_SIZE_L;
			set_cursor(TRUE);
			SendMessage(game_wnd, WM_SET_BOARD_SIZE, 0, BOARD_SIZE_L);
			init_window_size(hWnd);
			// EBhËʒuݒ
			if (IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
				SetWindowPos(hWnd, 0, 0, 0, window_width, window_height, SWP_NOMOVE | SWP_NOZORDER);
			}
			set_window_size(hWnd);
			SendMessage(game_wnd, WM_SIZE, 0, 0);
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, LOWORD(wParam), MF_BYCOMMAND);
			set_cursor(FALSE);
			break;

		case ID_MENUITEM_BOARD_M:
			// {[hTCYύX ()
			op.board_size = BOARD_SIZE_M;
			set_cursor(TRUE);
			SendMessage(game_wnd, WM_SET_BOARD_SIZE, 0, BOARD_SIZE_M);
			init_window_size(hWnd);
			// EBhËʒuݒ
			if (IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
				SetWindowPos(hWnd, 0, 0, 0, window_width, window_height, SWP_NOMOVE | SWP_NOZORDER);
			}
			set_window_size(hWnd);
			SendMessage(game_wnd, WM_SIZE, 0, 0);
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, LOWORD(wParam), MF_BYCOMMAND);
			set_cursor(FALSE);
			break;

		case ID_MENUITEM_BOARD_S:
			// {[hTCYύX ()
			op.board_size = BOARD_SIZE_S;
			set_cursor(TRUE);
			SendMessage(game_wnd, WM_SET_BOARD_SIZE, 0, BOARD_SIZE_S);
			init_window_size(hWnd);
			// EBhËʒuݒ
			if (IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
				SetWindowPos(hWnd, 0, 0, 0, window_width, window_height, SWP_NOMOVE | SWP_NOZORDER);
			}
			set_window_size(hWnd);
			SendMessage(game_wnd, WM_SIZE, 0, 0);
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, LOWORD(wParam), MF_BYCOMMAND);
			set_cursor(FALSE);
			break;

		case ID_MENUITEM_BOARD_HIDE:
			// {[h\
			op.board_size = 0;
			set_cursor(TRUE);
			SendMessage(game_wnd, WM_SET_BOARD_SIZE, 0, 0);
			init_window_size(hWnd);
			// EBhËʒuݒ
			if (IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
				SetWindowPos(hWnd, 0, 0, 0, window_width, window_height, SWP_NOMOVE | SWP_NOZORDER);
			}
			set_window_size(hWnd);
			SendMessage(game_wnd, WM_SIZE, 0, 0);
			CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_BOARD_L,
				ID_MENUITEM_BOARD_HIDE, LOWORD(wParam), MF_BYCOMMAND);
			set_cursor(FALSE);
			break;

		case ID_MENUITEM_STATUS_BAR:
			// Xe[^Xo[
			if (IsWindowVisible(status_wnd) != FALSE) {
				op.window_status_bar = 0;
				ShowWindow(status_wnd, SW_HIDE);
				CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_STATUS_BAR, MF_UNCHECKED);
			} else {
				op.window_status_bar = 1;
				ShowWindow(status_wnd, SW_SHOW);
				CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1), ID_MENUITEM_STATUS_BAR, MF_CHECKED);
			}
			init_window_size(hWnd);
			set_window_size(hWnd);
			break;

		case ID_MENUITEM_SCORE_LIST:
			// XRAꗗ
			SendMessage(game_wnd, WM_SHOW_SCORE, -1, 0);
			break;

		case ID_MENUITEM_SAVE_SCORE:
			// XRACSVŕۑ
			SendMessage(game_wnd, WM_SAVE_SCORE, 0, 0);
			break;

		case ID_MENUITEM_ABOUT:
			// o[W
			MessageBox(hWnd,
				APP_NAME
				TEXT("\nCopyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.\n\n")
				TEXT("WEB SITE: http://www.nakka.com/\nE-MAIL: nakka@nakka.com"),
				TEXT("About"), MB_OK | MB_ICONINFORMATION);
			break;
		}
		break;

	case WM_GAME_END:
		// Q[Iݒ
		SendMessage(game_wnd, WM_SET_BOARD_ENABLE, FALSE, 0);
		SendMessage(game_wnd, WM_SET_INPUT_ENABLE, FALSE, 0);
		set_enable_menu(hWnd, FALSE);
		SetFocus(NULL);
		game_flag = FALSE;
		break;

	case WM_SET_STATUS:
		statusbar_set_text(status_wnd, (TCHAR *)lParam);
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * commnad_line_func - R}hC
 */
static void commnad_line_func(void)
{
	TCHAR *p;

	p = GetCommandLine();
	// st@C̏
    if (*p == TEXT('"')) {
		for (p++; *p != TEXT('\0') && *p != TEXT('"'); p++)
			;
		if (*p != TEXT('\0')) {
			p++;
		}
	} else {
		for (; *p != TEXT('\0') && *p != TEXT(' '); p++)
			;
	}

	for (; *p != TEXT('\0') && *p != TEXT('/') && *p != TEXT('-'); p++)
		;
	if (*p == TEXT('\0')) {
		return;
	}
	for (p++; *p != TEXT('\0'); p++) {
		if (*p == TEXT(' ')) {
			for (; *p != TEXT('\0') && *p != TEXT('/') && *p != TEXT('-'); p++)
				;
			if (*p == TEXT('\0') || *(++p) == TEXT('\0')) {
				break;
			}
		}
		switch (*p) {
		case TEXT('l'): case TEXT('L'):
			// {[hTCY L
			op.board_size = BOARD_SIZE_L;
			break;

		case TEXT('m'): case TEXT('M'):
			// {[hTCY M
			op.board_size = BOARD_SIZE_M;
			break;

		case TEXT('s'): case TEXT('S'):
			// {[hTCY S
			op.board_size = BOARD_SIZE_S;
			break;

		case TEXT('h'): case TEXT('H'):
			// {[h\
			op.board_size = 0;
			break;

		case TEXT('z'): case TEXT('Z'):
			// 01Q[Jn
			init_game = GAME_TYPE_01;
			break;

		case TEXT('c'): case TEXT('C'):
			// NPbgJn
			init_game = GAME_TYPE_CRICKET;
			break;

		case TEXT('u'): case TEXT('U'):
			// JEgAbvJn
			init_game = GAME_TYPE_COUNTUP;
			break;

		case TEXT('k'): case TEXT('K'):
			// NPbg JEgAbvJn
			init_game = GAME_TYPE_CRICKET_COUNTUP;
			break;

		case TEXT('e'): case TEXT('E'):
			// C[OY ACJn
			init_game = GAME_TYPE_EAGLESEYE;
			break;

		case TEXT('i'): case TEXT('I'):
			// n[tCbgJn
			init_game = GAME_TYPE_HALFIT;
			break;

		case TEXT('r'): case TEXT('R'):
			// [e[VJn
			init_game = GAME_TYPE_ROTATION;
			break;

		case TEXT('t'): case TEXT('T'):
			// XvbgJn
			init_game = GAME_TYPE_SPLAT;
			break;
		}
	}
}

/*
 * get_path - pX̍쐬
 */
static void get_path(const HINSTANCE hInstance)
{
	TCHAR work_path[MAX_PATH];
	TCHAR *p, *r;

	// AvP[ṼpX擾
	GetModuleFileName(hInstance, work_path, MAX_PATH - 1);
	for (p = r = work_path; *p != TEXT('\0'); p++) {
#ifndef UNICODE
		if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
			p++;
			continue;
		}
#endif	// UNICODE
		if (*p == TEXT('\\') || *p == TEXT('/')) {
			r = p;
		}
	}
	*r = TEXT('\0');
	// inipX쐬
	wsprintf(ini_path, TEXT("%s\\%s"), work_path, INI_FILE);
}

/*
 * InitApplication - EBhENX̓o^
 */
static BOOL InitApplication(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.lpszMenuName = MAKEINTRESOURCE(IDR_MENU_MAIN);
	wc.lpfnWndProc = (WNDPROC)main_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_MAIN));
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszClassName = MAIN_WND_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * InitInstance - EBhE̍쐬
 */
static HWND InitInstance(const HINSTANCE hInstance, const int CmdShow)
{
	HWND hWnd =  NULL;

	// EBhE̍쐬
	hWnd = CreateWindowEx(0,
		MAIN_WND_CLASS,
		WINDOW_TITLE,
		WS_OVERLAPPEDWINDOW,
		op.window_rect.left,
		op.window_rect.top,
		op.window_rect.right,
		op.window_rect.bottom,
		NULL, NULL, hInstance, NULL);
	if (hWnd == NULL) {
		return NULL;
	}
	// EBhE̕\
	ShowWindow(hWnd, CmdShow);
	UpdateWindow(hWnd);
	return hWnd;
}

/*
 * WinMain - C
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	HWND hWnd;
	HANDLE hAccel;

	hInst = hInstance;

	if (stricmp(lpCmdLine, "/shortcut") == 0) {
		LPMALLOC pMalloc;
		LPITEMIDLIST pItemIDList;
		TCHAR app_path[MAX_PATH];
		TCHAR sp_path[MAX_PATH];
		TCHAR link_path[MAX_PATH];

		//OLE
		CoInitialize(NULL);
		// AvP[ṼpX擾
		GetModuleFileName(hInstance, app_path, MAX_PATH - 1);
		// fXNgbṽpX擾
		*sp_path = TEXT('\0');
		if (SHGetMalloc(&pMalloc) == NOERROR) {
			if (SHGetSpecialFolderLocation(NULL, CSIDL_COMMON_DESKTOPDIRECTORY, &pItemIDList) == NOERROR) {
				SHGetPathFromIDList(pItemIDList, sp_path);
				pMalloc->lpVtbl->Free(pMalloc, pItemIDList);
			}
			pMalloc->lpVtbl->Release(pMalloc);
		}
		if (*sp_path == TEXT('\0')) {
			if (SHGetMalloc(&pMalloc) == NOERROR) {
				if (SHGetSpecialFolderLocation(NULL, CSIDL_DESKTOP, &pItemIDList) == NOERROR) {
					SHGetPathFromIDList(pItemIDList, sp_path);
					pMalloc->lpVtbl->Free(pMalloc, pItemIDList);
				}
				pMalloc->lpVtbl->Release(pMalloc);
			}
		}
		if (*sp_path != TEXT('\0')) {
			if (*(sp_path + lstrlen(sp_path) - 1) == TEXT('\\')) {
				wsprintf(link_path, TEXT("%s%s"), sp_path, LNK_FILE);
			} else {
				wsprintf(link_path, TEXT("%s\\%s"), sp_path, LNK_FILE);
			}
			// V[gJbgt@C̍쐬
			create_shortcut(app_path, link_path);
		}
		CoUninitialize();
		return 0;
	}

	// ̏
	srand((unsigned)GetTickCount());

	// CommonControl̏
	InitCommonControls();

	// ݒ擾
	get_path(hInst);
	ini_get_option(ini_path);

	// EBhENX̓o^
	if (game_regist(hInstance) == FALSE ||
		board_regist(hInstance) == FALSE ||
		score_regist(hInstance) == FALSE ||
		score_01_regist(hInstance) == FALSE ||
		score_cricket_regist(hInstance) == FALSE ||
		score_countup_regist(hInstance) == FALSE ||
		score_cricket_countup_regist(hInstance) == FALSE ||
		score_eagleseye_regist(hInstance) == FALSE ||
		score_halfit_regist(hInstance) == FALSE ||
		score_rotation_regist(hInstance) == FALSE ||
		score_splat_regist(hInstance) == FALSE) {
		return 0;
	}
	if (InitApplication(hInstance) == FALSE) {
		return 0;
	}
	// EBhE̍쐬
	if ((hWnd = InitInstance(hInstance,  op.window_state)) == NULL) {
		return 0;
	}
	hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR));
	// EBhEbZ[W
	while (GetMessage(&msg, NULL, 0, 0) == TRUE) {
		if (TranslateAccelerator(hWnd, hAccel, &msg) == TRUE) {
			continue;
		}
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
#ifdef _DEBUG
	mem_debug();
#endif
	return msg.wParam;
}
/* End of source */
