/*
 * nScore
 *
 * score.c
 *
 * Copyright (C) 1996-2004 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE

#include "General.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "font.h"
#include "game.h"
#include "board.h"
#include "score.h"
#include "score_01.h"
#include "score_cricket.h"
#include "score_countup.h"
#include "score_cricket_countup.h"
#include "score_eagleseye.h"
#include "score_halfit.h"
#include "score_rotation.h"
#include "score_splat.h"
#include "score_input.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("score_wnd")

/* Global Variables */
static HWND score_01_wnd;
static HWND score_cricket_wnd;
static HWND score_countup_wnd;
static HWND score_cricket_countup_wnd;
static HWND score_eagleseye_wnd;
static HWND score_halfit_wnd;
static HWND score_rotation_wnd;
static HWND score_splat_wnd;

extern HINSTANCE hInst;
extern COLOR_INFO ci;

/* Local Function Prototypes */
static HWND get_score_window(const HWND hWnd);
static void hide_window(const HWND hWnd);
static LRESULT CALLBACK score_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_score_window - \EBhE擾
 */
static HWND get_score_window(const HWND hWnd)
{
	HWND cWnd;

	cWnd = GetWindow(hWnd, GW_CHILD);
	while (cWnd != NULL) {
		if (IsWindowVisible(cWnd) == TRUE) {
			break;
		}
		cWnd = GetNextWindow(cWnd, GW_HWNDNEXT);
	}
	return cWnd;
}

/*
 * hide_window - SEBhE\
 */
static void hide_window(const HWND hWnd)
{
	HWND cWnd;

	cWnd = GetWindow(hWnd, GW_CHILD);
	while (cWnd != NULL) {
		ShowWindow(cWnd, SW_HIDE);
		cWnd = GetNextWindow(cWnd, GW_HWNDNEXT);
	}
}

/*
 * score_proc - XRAEBhEvV[W
 */
static LRESULT CALLBACK score_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	TEXTMETRIC tm;
	HDC hdc;
	PAINTSTRUCT ps;
	HBRUSH draw_brush;
	HFONT hfont, hRetFont;
	RECT client_rect;
	HWND child_wnd;
	int height;
	static TCHAR d_msg[BUF_SIZE];

	switch (msg) {
	case WM_CREATE:
		// EBhE̍擾
		hdc = GetDC(hWnd);
		hfont = font_create(FONT_NAME, 24, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(hdc, hfont);
		GetTextMetrics(hdc, &tm);
		SelectObject(hdc, hRetFont);
		DeleteObject(hfont);
		ReleaseDC(hWnd, hdc);
		height = 35 + (tm.tmHeight + 5) * 7 + 5;

		// EBhE̍쐬
		score_01_wnd = score_01_create(hInst, hWnd, 0, height);
		score_cricket_wnd = score_cricket_create(hInst, hWnd, 0, height);
		score_countup_wnd = score_countup_create(hInst, hWnd, 0, height);
		score_cricket_countup_wnd = score_cricket_countup_create(hInst, hWnd, 0, height);
		score_eagleseye_wnd = score_eagleseye_create(hInst, hWnd, 0, height);
		score_halfit_wnd = score_halfit_create(hInst, hWnd, 0, height);
		score_rotation_wnd = score_rotation_create(hInst, hWnd, 0, height);
		score_splat_wnd = score_splat_create(hInst, hWnd, 0, height);

		// qEBhȆ傫ɍ킹
		GetClientRect(score_01_wnd, &client_rect);
		SetWindowPos(hWnd, 0, 0, 0, client_rect.right, client_rect.bottom, SWP_NOMOVE | SWP_NOZORDER);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// EBhE̔j
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		if (get_score_window(hWnd) == NULL) {
			break;
		}
		SetFocus(get_score_window(hWnd));
		break;

	case WM_SIZE:
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		// wi̓hԂ
		draw_brush = CreateSolidBrush(ci.background);
		FillRect(hdc, &ps.rcPaint, draw_brush);
		DeleteObject(draw_brush);
		// bZ[W̕`
		if (get_score_window(hWnd) == NULL) {
			SetTextColor(hdc, ci.text);
			SetBkColor(hdc, ci.background);
			GetClientRect(hWnd, &client_rect);
			DrawText(hdc, d_msg, lstrlen(d_msg), &client_rect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
		}
		EndPaint(hWnd, &ps);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case ID_ACCEL_RETURN:
		case IDC_BUTTON_NEXT:
			if (get_score_window(hWnd) == NULL) {
				break;
			}
			SendMessage(get_score_window(hWnd), WM_COMMAND, wParam, lParam);
			break;
		}
		break;

	case WM_SET_DRAW_MESSAGE:
		lstrcpy(d_msg, (TCHAR *)lParam);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SET_STATUS:
	case WM_GAME_END:
	case WM_SET_BOARD_ENABLE:
	case WM_SET_BULL_TYPE:
		return SendMessage(GetParent(hWnd), msg, wParam, lParam);

	case WM_GAME_STOP:
	case WM_SHOW_SCORE:
	case WM_SAVE_SCORE:
	case WM_PREV_PLAYER:
	case WM_SET_SCORE:
	case WM_SET_INPUT_ENABLE:
		if (get_score_window(hWnd) == NULL) {
			break;
		}
		return SendMessage(get_score_window(hWnd), msg, wParam, lParam);

	case WM_GAME_INIT:
		// Q[̏
		child_wnd = NULL;
		switch (wParam) {
		case GAME_TYPE_01:
			// 01Q[
			child_wnd = score_01_wnd;
			break;
		case GAME_TYPE_CRICKET:
			// NPbg
			child_wnd = score_cricket_wnd;
			break;
		case GAME_TYPE_COUNTUP:
			// JEgAbv
			child_wnd = score_countup_wnd;
			break;
		case GAME_TYPE_CRICKET_COUNTUP:
			// NPbg JEgAbv
			child_wnd = score_cricket_countup_wnd;
			break;
		case GAME_TYPE_EAGLESEYE:
			// C[OY AC
			child_wnd = score_eagleseye_wnd;
			break;
		case GAME_TYPE_HALFIT:
			// n[tCbg
			child_wnd = score_halfit_wnd;
			break;
		case GAME_TYPE_ROTATION:
			// [e[V
			child_wnd = score_rotation_wnd;
			break;
		case GAME_TYPE_SPLAT:
			// Xvbg
			child_wnd = score_splat_wnd;
			break;
		}
		if (child_wnd == NULL) {
			break;
		}
		// ݒ
		if (SendMessage(child_wnd, WM_GAME_OPTION, 0, 0) == FALSE) {
			return FALSE;
		}
		// EBhE\ݒ
		hide_window(hWnd);
		ShowWindow(child_wnd, SW_SHOW);
		SendMessage(hWnd, WM_SET_BOARD_ENABLE, TRUE, 0);
		SendMessage(hWnd, WM_SET_INPUT_ENABLE, TRUE, 0);
		// 
		SendMessage(child_wnd, WM_GAME_INIT, 0, 0);
		return TRUE;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_regist - EBhENX̓o^
 */
BOOL score_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)score_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_create - EBhE̍쐬
 */
HWND score_create(const HINSTANCE hInstance, const HWND pWnd, int id)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_VISIBLE | WS_TABSTOP,
		0, 0, SCORE_WIDTH, SCORE_HEIGHT, pWnd, (HMENU)id, hInstance, NULL);
	return hWnd;
}
/* End of source */
