/*
 * nScore
 *
 * score_01.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <stdio.h>

#include "General.h"
#include "Memory.h"
#include "string.h"
#include "Message.h"
#include "file.h"
#include "font.h"
#include "game.h"
#include "board.h"
#include "score.h"
#include "score_01.h"
#include "score_01_list.h"
#include "score_01_option.h"
#include "score_input.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("score_01_wnd")
#define ERROR_TITLE					TEXT("nScore - Error")

#define WM_DRAW_SCORE				(WM_APP + 400)

/* Global Variables */
ZEROONE_INFO g_zi;
static HWND input_wnd;

extern HINSTANCE hInst;
extern COLOR_INFO ci;
extern BYTE char_set;

typedef struct _BUFFER {
	HBITMAP hbmp;

	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;
	HBRUSH draw_brush;
} BUFFER;

static BUFFER bf;

/* Local Function Prototypes */
static BOOL get_arrange(const int score, TCHAR *buf);
static int get_score_string_length(const ZEROONE_INFO *zi);
static BOOL get_score_string(const ZEROONE_INFO *zi, TCHAR *ret);
static BOOL score_save(const HWND hWnd, const ZEROONE_INFO *zi);
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const ZEROONE_INFO *zi);
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const ZEROONE_INFO *zi, const int score, const TCHAR *arrange);
static BOOL result_draw(const BUFFER *draw_bf, const RECT *rect, const ZEROONE_INFO *zi);
static LRESULT CALLBACK score_01_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_arrange - AW̕`
 */
static BOOL get_arrange(const int score, TCHAR *buf)
{
	TCHAR tmp[BUF_SIZE];
	TCHAR *p;
	int i;
	int arrange_tbl[171][THROW_MAX] = {
		{0,	0, 0},			// 0
		{0, 0, 0},			// 1
		{20 + 1, 0, 0},		// 2
		{1, 20 + 1, 0},		// 3
		{20 + 2, 0, 0},		// 4
		{1, 20 + 2, 0},		// 5
		{20 + 3, 0, 0},		// 6
		{3, 20 + 2, 0},		// 7
		{20 + 4, 0, 0},		// 8
		{1, 20 + 4, 0},		// 9

		{20 + 5, 0, 0},		// 10
		{3, 20 + 4, 0},		// 11
		{20 + 6, 0, 0},		// 12
		{5, 20 + 4, 0},		// 13
		{20 + 7, 0, 0},		// 14
		{7, 20 + 4, 0},		// 15
		{20 + 8, 0, 0},		// 16
		{1, 20 + 8, 0},		// 17
		{20 + 9, 0, 0},		// 18
		{3, 20 + 8, 0},		// 19

		{20 + 10, 0, 0},	// 20
		{5, 20 + 8, 0},		// 21
		{20 + 11, 0, 0},	// 22
		{7, 20 + 8, 0},		// 23
		{20 + 12, 0, 0},	// 24
		{17, 20 + 4, 0},	// 25
		{20 + 13, 0, 0},	// 26
		{19, 20 + 4, 0},	// 27
		{20 + 14, 0, 0},	// 28
		{13, 20 + 8, 0},	// 29

		{20 + 15, 0, 0},	// 30
		{15, 20 + 8, 0},	// 31
		{20 + 16, 0, 0},	// 32
		{1, 20 + 16, 0},	// 33
		{20 + 17, 0, 0},	// 34
		{3, 20 + 16, 0},	// 35
		{20 + 18, 0, 0},	// 36
		{5, 20 + 16, 0},	// 37
		{20 + 19, 0, 0},	// 38
		{7, 20 + 16, 0},	// 39

		{20 + 20, 0, 0},	// 40
		{9, 20 + 16, 0},	// 41
		{10, 20 + 16, 0},	// 42
		{3, 20 + 20, 0},	// 43
		{12, 20 + 16, 0},	// 44
		{13, 20 + 16, 0},	// 45
		{6, 20 + 20, 0},	// 46
		{7, 20 + 20, 0},	// 47
		{16, 20 + 16, 0},	// 48
		{17, 20 + 16, 0},	// 49

		{62, 0, 0},			// 50
		{19, 20 + 16, 0},	// 51
		{12, 20 + 20, 0},	// 52
		{13, 20 + 20, 0},	// 53
		{14, 20 + 20, 0},	// 54
		{15, 20 + 20, 0},	// 55
		{16, 20 + 20, 0},	// 56
		{17, 20 + 20, 0},	// 57
		{18, 20 + 20, 0},	// 58
		{19, 20 + 20, 0},	// 59

		{20, 20 + 20, 0},		// 60
		{40 + 15, 20 + 8, 0},	// 61
		{40 + 10, 20 + 16, 0},	// 62
		{40 + 17, 20 + 6, 0},	// 63
		{40 + 16, 20 + 8, 0},	// 64
		{40 + 19, 20 + 4, 0},	// 65
		{40 + 10, 20 + 18, 0},	// 66
		{40 + 17, 20 + 8, 0},	// 67
		{40 + 20, 20 + 4, 0},	// 68
		{40 + 19, 20 + 6, 0},	// 69

		{40 + 18, 20 + 8, 0},	// 70
		{40 + 13, 20 + 16, 0},	// 71
		{40 + 12, 20 + 18, 0},	// 72
		{40 + 19, 20 + 8, 0},	// 73
		{40 + 14, 20 + 16, 0},	// 74
		{61, 62, 0},			// 75
		{40 + 20, 20 + 8, 0},	// 76
		{40 + 19, 20 + 10, 0},	// 77
		{40 + 18, 20 + 12, 0},	// 78
		{40 + 19, 20 + 11, 0},	// 79

		{40 + 20, 20 + 10, 0},	// 80
		{40 + 15, 20 + 18, 0},	// 81
		{62, 20 + 16, 0},		// 82
		{40 + 17, 20 + 16, 0},	// 83
		{40 + 20, 20 + 12, 0},	// 84
		{40 + 15, 20 + 20, 0},	// 85
		{40 + 18, 20 + 16, 0},	// 86
		{40 + 17, 20 + 18, 0},	// 87
		{40 + 20, 20 + 14, 0},	// 88
		{40 + 19, 20 + 16, 0},	// 89

		{20, 20, 62},			// 90
		{40 + 17, 20 + 20, 0},	// 91
		{40 + 20, 20 + 16, 0},	// 92
		{40 + 19, 20 + 18, 0},	// 93
		{40 + 18, 20 + 20, 0},	// 94
		{40 + 19, 20 + 19, 0},	// 95
		{40 + 20, 20 + 18, 0},	// 96
		{40 + 19, 20 + 20, 0},	// 97
		{40 + 20, 20 + 19, 0},	// 98
		{40 + 19, 10, 20 + 16},	// 99

		{40 + 20, 20 + 20, 0},	// 100
		{40 + 17, 62, 0},		// 101
		{40 + 20, 10, 20 + 16},	// 102
		{40 + 20, 3, 20 + 20},	// 103
		{40 + 18, 62, 0},		// 104
		{40 + 19, 16, 20 + 16},	// 105
		{40 + 20, 6, 20 + 20},	// 106
		{40 + 19, 62, 0},		// 107
		{40 + 20, 16, 20 + 16},	// 108
		{40 + 20, 17, 20 + 16},	// 109

		{40 + 20, 62, 0},		// 110
		{40 + 20, 19, 20 + 16},	// 111
		{40 + 20, 12, 20 + 20},	// 112
		{40 + 20, 13, 20 + 20},	// 113
		{40 + 20, 14, 20 + 20},	// 114
		{40 + 19, 18, 20 + 20},	// 115
		{40 + 20, 16, 20 + 20},	// 116
		{40 + 20, 17, 20 + 20},	// 117
		{40 + 18, 14, 62},		// 118
		{19, 40 + 20, 20 + 20},	// 119

		{40 + 20, 20, 20 + 20},	// 120
		{40 + 20, 11, 62},		// 121
		{40 + 18, 18, 62},		// 122
		{40 + 19, 16, 62},		// 123
		{40 + 20, 14, 62},		// 124
		{61, 40 + 20, 20 + 20},	// 125
		{40 + 19, 19, 62},		// 126
		{40 + 20, 17, 62},		// 127
		{18, 40 + 20, 62},		// 128
		{40 + 19, 40 + 12, 20 + 18},	// 129

		{40 + 20, 20, 62},				// 130
		{40 + 20, 40 + 13, 20 + 16},	// 131
		{40 + 20, 40 + 12, 20 + 18},	// 132
		{40 + 20, 40 + 19, 20 + 8},		// 133
		{40 + 20, 40 + 14, 20 + 16},	// 134
		{40 + 20, 61, 62},				// 135
		{40 + 20, 40 + 20, 20 + 8},		// 136
		{40 + 20, 40 + 19, 20 + 10},	// 137
		{40 + 20, 40 + 18, 20 + 12},	// 138
		{40 + 20, 40 + 13, 20 + 20},	// 139

		{40 + 20, 40 + 20, 20 + 10},	// 140
		{40 + 20, 40 + 15, 20 + 18},	// 141
		{40 + 20, 40 + 14, 20 + 20},	// 142
		{40 + 20, 40 + 17, 20 + 16},	// 143
		{40 + 20, 40 + 20, 20 + 12},	// 144
		{40 + 20, 40 + 15, 20 + 20},	// 145
		{40 + 20, 40 + 18, 20 + 16},	// 146
		{40 + 20, 40 + 17, 20 + 18},	// 147
		{40 + 20, 40 + 20, 20 + 14},	// 148
		{40 + 20, 40 + 19, 20 + 16},	// 149

		{40 + 20, 40 + 18, 20 + 18},	// 150
		{40 + 20, 40 + 17, 20 + 20},	// 151
		{40 + 20, 40 + 20, 20 + 16},	// 152
		{40 + 20, 40 + 19, 20 + 18},	// 153
		{40 + 20, 40 + 18, 20 + 20},	// 154
		{40 + 20, 40 + 19, 20 + 19},	// 155
		{40 + 20, 40 + 20, 20 + 18},	// 156
		{40 + 20, 40 + 19, 20 + 20},	// 157
		{40 + 20, 40 + 20, 20 + 19},	// 158
		{0, 0, 0},						// 159

		{40 + 20, 40 + 20, 20 + 20},	// 160
		{40 + 20, 40 + 17, 62},			// 161
		{0, 0, 0},						// 162
		{0, 0, 0},						// 163
		{40 + 20, 40 + 18, 62},			// 164
		{0, 0, 0},						// 165
		{0, 0, 0},						// 166
		{40 + 20, 40 + 19, 62},			// 167
		{0, 0, 0},						// 168
		{0, 0, 0},						// 169

		{40 + 20, 40 + 20, 62},			// 170
	};

/*	{
		int i, j, s;

		for (i = 0; i < 171; i++) {
			s = 0;
			for (j = 0; j < THROW_MAX; j++) {
				if (arrange_tbl[i][j] <= 0) {
				} else if (arrange_tbl[i][j] <= 20) {
					s += arrange_tbl[i][j];
				} else if (arrange_tbl[i][j] <= 40) {
					s += (arrange_tbl[i][j] - 20) * 2;
				} else if (arrange_tbl[i][j] <= 60) {
					s += (arrange_tbl[i][j] - 40) * 3;
				} else if (arrange_tbl[i][j] == 61) {
					s += 25;
				} else if (arrange_tbl[i][j] == 62) {
					s += 50;
				}
			}
			if (s != i && s != 0) {
				wsprintf(tmp, TEXT("%d"), i);
				MessageBox(NULL, tmp, TEXT("Arrange Error"), 0);
			}
		}
	}*/

	*buf = TEXT('\0');
	p = buf;

	if (score <= 1 || score > 170) {
		return FALSE;
	}
	for (i = 0; i < THROW_MAX; i++) {
		*tmp = TEXT('\0');
		if (arrange_tbl[score][i] <= 0) {
		} else if (arrange_tbl[score][i] <= 20) {
			wsprintf(tmp, TEXT("%s %d"), message_get_res(IDS_STRING_SINGLE), arrange_tbl[score][i]);
		} else if (arrange_tbl[score][i] <= 40) {
			wsprintf(tmp, TEXT("%s %d"), message_get_res(IDS_STRING_DOUBLE), arrange_tbl[score][i] - 20);
		} else if (arrange_tbl[score][i] <= 60) {
			wsprintf(tmp, TEXT("%s %d"), message_get_res(IDS_STRING_TRIPLE), arrange_tbl[score][i] - 40);
		} else if (arrange_tbl[score][i] == 61) {
			lstrcpy(tmp, message_get_res(IDS_STRING_S_BULL));
		} else if (arrange_tbl[score][i] == 62) {
			lstrcpy(tmp, message_get_res(IDS_STRING_D_BULL));
		}
		if (*tmp != TEXT('\0')) {
			if (*buf != TEXT('\0')) {
				lstrcpy(p, TEXT(", "));
				p += 2;
			}
			lstrcpy(p, tmp);
			p += lstrlen(tmp);
		}
	}
	return TRUE;
}

/*
 * get_01_average - ώ擾
 */
float get_01_average(const ZEROONE_INFO *zi, const int player, float *ret1, float *ret2)
{
	float avg, avg1, avg2;
	int round_score;
	int cnt;
	int i, j, j1, j2;
	BOOL d_in;

	cnt = zi->current_round;
	if ((zi->current_player <= player && (zi->game_flag == TRUE || zi->current_player != player)) ||
		(cnt > zi->round_cnt && player == 0)) {
		cnt--;
	}
	avg = avg1 = avg2 = 0;

	d_in = zi->pi[player].double_in;
	j = j1 = j2 = 0;
	for (i = 0; i < cnt; i++) {
		if (zi->average_round > 0 && ((zi->average_round == i + 1 && zi->average_player < player) || zi->average_round < i + 1)) {
			break;
		}

		if (i == 0) {
			round_score = zi->init_score - zi->pi[player].ri[i].score_list;
		} else {
			round_score = zi->pi[player].ri[i - 1].score_list - zi->pi[player].ri[i].score_list;
		}
		if (d_in == TRUE && round_score == 0) {
			continue;
		} else {
			d_in = FALSE;
		}

		avg += round_score;
		j++;
		if (i == 0 || i / 2 == (i + 1) / 2) {
			avg1 += round_score;
			j1++;
		} else {
			avg2 += round_score;
			j2++;
		}
	}
	if (j > 0) {
		avg = (float)avg / (float)j;
	} else {
		avg = 0;
	}
	if (j1 > 0) {
		avg1 = (float)avg1 / (float)j1;
	} else {
		avg1 = 0;
	}
	if (j2 > 0) {
		avg2 = (float)avg2 / (float)j2;
	} else {
		avg2 = 0;
	}

	if (ret1 != NULL) {
		*ret1 = avg1;
	}
	if (ret2 != NULL) {
		*ret2 = avg2;
	}
	return avg;
}

/*
 * get_score_string_length - XRA̒擾
 */
static int get_score_string_length(const ZEROONE_INFO *zi)
{
	TCHAR buf[BUF_SIZE];
	int i, j, t;
	int ret = 0;

	wsprintf(buf, TEXT("%d"), zi->init_score);
	ret += lstrlen(buf);
	// O
	for (i = 0; i < zi->player_cnt; i++) {
		_stprintf(buf, TEXT(",%s,%.2f,,,"), zi->pi[i].name, get_01_average(zi, i, NULL, NULL));
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// ^Cg
	ret += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < zi->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			ret += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_ROUND_SCORE));
		ret += lstrlen(buf);
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// XRA
	for (j = 0; j < zi->current_round; j++) {
		if (j == zi->current_round - 1 && zi->current_player == 0 && zi->game_flag == TRUE) {
			break;
		}
		if (j >= zi->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		ret += lstrlen(buf);
		for (i = 0; i < zi->player_cnt; i++) {
			// throw
			for (t = 0; t < THROW_MAX; t++) {
				ret++;		//,
				ret += lstrlen(zi->pi[i].ri[j].throw_list[t]);
			}
			if (j == zi->current_round - 1 &&
				(i > zi->current_player || (zi->game_flag == TRUE && i == zi->current_player))) {
				ret++;		//,
				ret++;		//,
				continue;
			}
			// Round score
			if (j == 0) {
				wsprintf(buf, TEXT(",%d"), zi->init_score - zi->pi[i].ri[j].score_list);
			} else {
				wsprintf(buf, TEXT(",%d"), zi->pi[i].ri[j - 1].score_list - zi->pi[i].ri[j].score_list);
			}
			ret += lstrlen(buf);
			// Score
			wsprintf(buf, TEXT(",%d"), zi->pi[i].ri[j].score_list);
			ret += lstrlen(buf);
		}
		ret += 2;	//CRLF
	}
	return ret;
}

/*
 * get_score_string - XRA̍쐬
 */
static BOOL get_score_string(const ZEROONE_INFO *zi, TCHAR *ret)
{
	TCHAR *p = ret;
	TCHAR buf[BUF_SIZE];
	int i, j, t;

	wsprintf(buf, TEXT("%d"), zi->init_score);
	lstrcpy(p, buf);
	p += lstrlen(buf);
	// O
	for (i = 0; i < zi->player_cnt; i++) {
		_stprintf(buf, TEXT(",%s,%.2f,,,"), zi->pi[i].name, get_01_average(zi, i, NULL, NULL));
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// ^Cg
	lstrcpy(p, message_get_res(IDS_STRING_LIST_ROUND));
	p += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < zi->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_ROUND_SCORE));
		lstrcpy(p, buf);
		p += lstrlen(buf);
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// XRA
	for (j = 0; j < zi->current_round; j++) {
		if (j == zi->current_round - 1 && zi->current_player == 0 && zi->game_flag == TRUE) {
			break;
		}
		if (j >= zi->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		lstrcpy(p, buf);
		p += lstrlen(buf);
		for (i = 0; i < zi->player_cnt; i++) {
			// throw
			for (t = 0; t < THROW_MAX; t++) {
				*(p++) = TEXT(',');
				lstrcpy(p, zi->pi[i].ri[j].throw_list[t]);
				p += lstrlen(zi->pi[i].ri[j].throw_list[t]);
			}
			if (j == zi->current_round - 1 &&
				(i > zi->current_player || (zi->game_flag == TRUE && i == zi->current_player))) {
				*(p++) = TEXT(',');
				*(p++) = TEXT(',');
				continue;
			}
			// Round score
			if (j == 0) {
				wsprintf(buf, TEXT(",%d"), zi->init_score - zi->pi[i].ri[j].score_list);
			} else {
				wsprintf(buf, TEXT(",%d"), zi->pi[i].ri[j - 1].score_list - zi->pi[i].ri[j].score_list);
			}
			lstrcpy(p, buf);
			p += lstrlen(buf);
			// Score
			wsprintf(buf, TEXT(",%d"), zi->pi[i].ri[j].score_list);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		*(p++) = TEXT('\r');
		*(p++) = TEXT('\n');
	}
	*p = TEXT('\0');
	return TRUE;
}

/*
 * score_save - XRA̕ۑ
 */
static BOOL score_save(const HWND hWnd, const ZEROONE_INFO *zi)
{
	TCHAR path[MAX_PATH];
	TCHAR *buf;
	TCHAR err_str[BUF_SIZE];
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	DWORD size;

	// t@C̐
	if (GetDateFormat(0, 0, NULL, TEXT("yyyyMMdd"), date, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	if (GetTimeFormat(0, 0, NULL, TEXT("HHmmss"), time, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	wsprintf(path, TEXT("%d_%s_%s.csv"), zi->init_score, date, time);

	// t@C̑I
	if (get_save_path(hWnd, message_get_res(IDS_STRING_SAVE_TITLE),
		TEXT("CSV (*.csv)\0*.csv\0All files (*.*)\0*.*\0\0"), TEXT("csv"), path) == FALSE) {
		return FALSE;
	}

	// XRA̍쐬
	size = get_score_string_length(zi);
	buf = mem_alloc(sizeof(TCHAR) * (size + 1));
	if (buf == NULL) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	get_score_string(zi, buf);

	if (file_write_buf(path, (BYTE *)buf, sizeof(TCHAR) * size, err_str) == FALSE) {
		mem_free(&buf);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	mem_free(&buf);
	return TRUE;
}

/*
 * round_draw - Eh̕`
 */
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const ZEROONE_INFO *zi)
{
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	int i;

	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	i = (zi->current_round > zi->round_cnt) ? zi->round_cnt : zi->current_round;
	wsprintf(buf, message_get_res(IDS_STRING_DRAW_ROUND), i, zi->round_cnt);
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), draw_rect, 0);
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * score_draw - XRA̕`
 */
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const ZEROONE_INFO *zi, const int score, const TCHAR *arrange)
{
	TEXTMETRIC tm;
	RECT draw_rect;
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	draw_rect.left += 10;
	draw_rect.right -= 10;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, zi);

	// 
	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_DRAW_AVERAGE), lstrlen(message_get_res(IDS_STRING_DRAW_AVERAGE)), &draw_rect, DT_RIGHT);
	draw_rect.top += tm.tmHeight;
	_stprintf(buf, TEXT("%.2f"), get_01_average(zi, zi->current_player, NULL, NULL));
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_RIGHT);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// vC[
	hfont = font_create(FONT_NAME, 20, char_set, 0, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	DrawText(draw_bf->draw_dc, zi->pi[zi->current_player].name, lstrlen(zi->pi[zi->current_player].name), &draw_rect, DT_CENTER);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight + 10;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// IvV\
	hfont = font_create(FONT_NAME, 20, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	*buf = TEXT('\0');
	if (zi->pi[zi->current_player].double_in == TRUE && zi->pi[zi->current_player].double_in_flag == 0) {
		lstrcpy(buf, message_get_res(IDS_STRING_DOUBLE_IN));
	} else if (zi->pi[zi->current_player].double_out == TRUE) {
		lstrcpy(buf, message_get_res(IDS_STRING_DOUBLE_OUT));
	}
	if (*buf != TEXT('\0')) {
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
	}
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// XRA
	hfont = font_create(SCORE_FONT_NAME, 100, 0, FW_HEAVY, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	if (score >= 0 && !(zi->pi[zi->current_player].double_out == TRUE && score == 1)) {
		_itot(score, buf, 10);
	} else {
		lstrcpy(buf, message_get_res(IDS_STRING_BUST));
	}
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// AW
	if (arrange != NULL && *arrange != TEXT('\0')) {
		hfont = font_create(FONT_NAME, 14, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		DrawText(draw_bf->draw_dc, arrange, lstrlen(arrange), &draw_rect, DT_CENTER);
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);
	}
	return TRUE;
}

/*
 * result_draw - ʂ̕`
 */
static BOOL result_draw(const BUFFER *draw_bf, const RECT *rect, const ZEROONE_INFO *zi)
{
	TEXTMETRIC tm;
	RECT draw_rect;
	HFONT hfont, hRetFont;
	HPEN pen, ret_pen;
	TCHAR buf[BUF_SIZE];
	int score_width;
	int top1, top2;
	int line_top1, line_top2;
	int j;
	float avg, avg1, avg2;

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	draw_rect.left += 10;
	draw_rect.right -= 10;
	score_width = draw_rect.right - draw_rect.left;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, zi);

	// vC[
	for (j = 0; j < zi->player_cnt; j++) {
		draw_rect.top = 35;
		draw_rect.left = j * (score_width / 4) + 22;
		draw_rect.right = draw_rect.left + 100;

		// ώ擾
		avg = get_01_average(zi, j, &avg1, &avg2);

		// Average title
		hfont = font_create(FONT_NAME, 12, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_DRAW_AVERAGE), lstrlen(message_get_res(IDS_STRING_DRAW_AVERAGE)), &draw_rect, DT_CENTER);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		draw_rect.top += tm.tmHeight + 5;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);
		// Average
		hfont = font_create(FONT_NAME, 20, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		_stprintf(buf, TEXT("%.2f"), avg);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		line_top1 = draw_rect.top + tm.tmHeight + 15;
		draw_rect.top += tm.tmHeight + 30;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);

		// First title & Second title
		hfont = font_create(FONT_NAME, 12, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_FIRST), lstrlen(message_get_res(IDS_STRING_FIRST)), &draw_rect, DT_CENTER);
		top1 = draw_rect.top + tm.tmHeight + 5;
		draw_rect.top += tm.tmHeight + 50;
		DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_SECOND), lstrlen(message_get_res(IDS_STRING_SECOND)), &draw_rect, DT_CENTER);
		top2 = draw_rect.top + tm.tmHeight + 5;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);
		// First rate & Second Average
		hfont = font_create(FONT_NAME, 14, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		draw_rect.top = top1;
		_stprintf(buf, TEXT("%.2f"), avg1);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		draw_rect.top = top2;
		_stprintf(buf, TEXT("%.2f"), avg2);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		line_top2 = draw_rect.top + tm.tmHeight + 15;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);

		//darts
		if (zi->pi[j].score == 0) {
			hfont = font_create(FONT_NAME, 14, 0, 0, FALSE, TRUE);
			hRetFont = SelectObject(draw_bf->draw_dc, hfont);
			GetTextMetrics(draw_bf->draw_dc, &tm);
			draw_rect.top = line_top2 + 15;
			wsprintf(buf, TEXT("%d darts"), zi->pi[j].darts);
			DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
			SelectObject(draw_bf->draw_dc, hRetFont);
			DeleteObject(hfont);
		}
	}
	// ̕`
	pen = CreatePen(PS_SOLID, 0, ci.line);
	ret_pen = SelectObject(draw_bf->draw_dc, pen);
	MoveToEx(draw_bf->draw_dc, rect->left + 15, line_top1, NULL);
	LineTo(draw_bf->draw_dc, rect->right - 15, line_top1);
	MoveToEx(draw_bf->draw_dc, rect->left + 15, line_top2, NULL);
	LineTo(draw_bf->draw_dc, rect->right - 15, line_top2);
	SelectObject(draw_bf->draw_dc, ret_pen);
	DeleteObject(pen);
	return TRUE;
}

/*
 * score_01_proc - EBhEvV[W
 */
static LRESULT CALLBACK score_01_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	RECT client_rect, input_rect;
	TCHAR buf[BUF_SIZE];
	int i, j;
	BOOL double_in_flag;
	BOOL bust_flag;

	switch (msg) {
	case WM_CREATE:
		// ̓EBhȄ
		input_wnd = score_input_create(hInst, hWnd);
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.right = input_rect.right;
		client_rect.bottom += input_rect.bottom;
		SetWindowPos(hWnd, 0, 0, 0, client_rect.right, client_rect.bottom, SWP_NOMOVE | SWP_NOZORDER);
		SetWindowPos(input_wnd, 0, 0, client_rect.bottom - input_rect.bottom, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER);

		// `p
		hdc = GetDC(hWnd);
		client_rect.bottom -= input_rect.bottom;
		bf.draw_dc = CreateCompatibleDC(hdc);
		bf.draw_bmp = CreateCompatibleBitmap(hdc, client_rect.right, client_rect.bottom);
		bf.draw_ret_bmp = SelectObject(bf.draw_dc, bf.draw_bmp);
		ReleaseDC(hWnd, hdc);
		bf.draw_brush = CreateSolidBrush(ci.background);
		FillRect(bf.draw_dc, &client_rect, bf.draw_brush);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// EBhE̔j
		SendMessage(input_wnd, WM_CLOSE, 0, 0);

		SelectObject(bf.draw_dc, bf.draw_ret_bmp);
		DeleteObject(bf.draw_bmp);
		DeleteDC(bf.draw_dc);
		DeleteObject(bf.draw_brush);
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		SetFocus(input_wnd);
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf.draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_LBUTTONDBLCLK:
		break;

	case WM_GAME_OPTION:
		// 01Q[ݒ
		return set_option_01(hInst, hWnd, &g_zi);

	case WM_GAME_INIT:
		// Q[̏
		g_zi.game_flag = TRUE;
		for (i = 0; i < g_zi.player_cnt; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)g_zi.pi[i].name);
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i, g_zi.pi[i].score);
		}
		for (; i < PLAYER_MAX; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)TEXT(""));
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, i, (LPARAM)TEXT(""));
		}
		g_zi.current_round = 1;
		g_zi.current_player = 0;
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_zi.pi[g_zi.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(GetParent(hWnd), WM_SET_BULL_TYPE, g_zi.op_bull, 0);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_zi.current_player);
		SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, -1);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		SendMessage(hWnd, WM_DRAW_SCORE, g_zi.pi[g_zi.current_player].score, 0);
		break;

	case WM_GAME_STOP:
		break;

	case WM_CHANGE_FOCUS:
		if (g_zi.game_flag == FALSE) {
			break;
		}
		if (g_zi.pi[g_zi.current_player].double_in == TRUE && g_zi.pi[g_zi.current_player].double_in_flag == 0) {
			// _uC
			i = 0;
			for (j = 0; j < THROW_MAX; j++) {
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf == CHAR_DOUBLE || double_in_flag == TRUE) {
					i += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
					double_in_flag = TRUE;
					if ((g_zi.pi[g_zi.current_player].double_out == TRUE && g_zi.pi[g_zi.current_player].score - i <= 1) ||
						g_zi.pi[g_zi.current_player].score - i <= 0) {
						break;
					}
				}
			}
		} else {
			// XRA擾
			i = 0;
			for (j = 0; j < THROW_MAX; j++) {
				i += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
				if ((g_zi.pi[g_zi.current_player].double_out == TRUE && g_zi.pi[g_zi.current_player].score - i <= 1) ||
					g_zi.pi[g_zi.current_player].score - i <= 0) {
					break;
				}
			}
		}
		bust_flag = TRUE;
		if (g_zi.pi[g_zi.current_player].double_out == TRUE && g_zi.pi[g_zi.current_player].score - i <= 1) {
			// _uAEg
			if (g_zi.pi[g_zi.current_player].score - i == 0) {
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf == CHAR_DOUBLE) {
					bust_flag = FALSE;
				}
			}
		} else if (g_zi.pi[g_zi.current_player].score - i >= 0) {
			bust_flag = FALSE;
		}
		if (bust_flag == FALSE) {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_zi.current_player, g_zi.pi[g_zi.current_player].score - i);
			SendMessage(hWnd, WM_DRAW_SCORE, g_zi.pi[g_zi.current_player].score - i, 0);
			if (g_zi.pi[g_zi.current_player].score - i == 0) {
				SendMessage(input_wnd, WM_SET_FOCUS_NEXT_BTN, 0, 0);
			}
		} else {
			// o[Xg
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_zi.current_player, g_zi.pi[g_zi.current_player].score);
			SendMessage(hWnd, WM_DRAW_SCORE, g_zi.pi[g_zi.current_player].score - i, 0);
			SendMessage(input_wnd, WM_SET_FOCUS_NEXT_BTN, 0, 0);
		}
		break;

	case WM_PREV_PLAYER:
		if (g_zi.current_player <= 0 && g_zi.current_round <= 1) {
			break;
		}
		SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_zi.current_player, g_zi.pi[g_zi.current_player].score);

		g_zi.current_player--;
		if (g_zi.current_player < 0) {
			g_zi.current_player = g_zi.player_cnt - 1;
			g_zi.current_round--;
		}
		if (g_zi.current_round <= 1) {
			g_zi.pi[g_zi.current_player].score = g_zi.init_score;
		} else {
			g_zi.pi[g_zi.current_player].score = g_zi.pi[g_zi.current_player].ri[g_zi.current_round - 1 - 1].score_list;
		}
		if (g_zi.pi[g_zi.current_player].double_in_flag == g_zi.current_round) {
			g_zi.pi[g_zi.current_player].double_in_flag = 0;
		}

		// ϑΏۃEh̏
		if (g_zi.average_round >= g_zi.current_round && g_zi.average_player >= g_zi.current_player) {
			g_zi.average_round = 0;
			g_zi.average_player = 0;
		}

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_zi.pi[g_zi.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_zi.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_zi.pi[g_zi.current_player].ri[g_zi.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_NEXT_PLAYER:
		if (g_zi.pi[g_zi.current_player].double_in == TRUE && g_zi.pi[g_zi.current_player].double_in_flag == 0) {
			// _uC
			i = 0;
			for (j = 0; j < THROW_MAX; j++) {
				SendMessage(input_wnd, WM_GET_SCORE_FULL_STR, j + 1, (LPARAM)g_zi.pi[g_zi.current_player].ri[g_zi.current_round - 1].throw_list[j]);
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf == CHAR_DOUBLE || g_zi.pi[g_zi.current_player].double_in_flag != 0) {
					i += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
					if (g_zi.pi[g_zi.current_player].double_in_flag == 0) {
						g_zi.pi[g_zi.current_player].double_in_flag = g_zi.current_round;
					}
					if ((g_zi.pi[g_zi.current_player].double_out == TRUE && g_zi.pi[g_zi.current_player].score - i <= 1) ||
						g_zi.pi[g_zi.current_player].score - i <= 0) {
						break;
					}
				}
			}
		} else {
			// XRA擾
			i = 0;
			for (j = 0; j < THROW_MAX; j++) {
				SendMessage(input_wnd, WM_GET_SCORE_FULL_STR, j + 1, (LPARAM)g_zi.pi[g_zi.current_player].ri[g_zi.current_round - 1].throw_list[j]);
				// XRA擾
				i += SendMessage(input_wnd, WM_GET_SCORE, j + 1, 0);
				if ((g_zi.pi[g_zi.current_player].double_out == TRUE && g_zi.pi[g_zi.current_player].score - i <= 1) ||
					g_zi.pi[g_zi.current_player].score - i <= 0) {
					break;
				}
			}
		}
		if (g_zi.pi[g_zi.current_player].double_out == TRUE && g_zi.pi[g_zi.current_player].score - i <= 1) {
			// _uAEg
			if (g_zi.pi[g_zi.current_player].score - i == 0) {
				SendMessage(input_wnd, WM_GET_SCORE_STR, j + 1, (LPARAM)buf);
				if (*buf == CHAR_DOUBLE) {
					g_zi.pi[g_zi.current_player].score -= i;
				}
			}
		} else if (g_zi.pi[g_zi.current_player].score - i >= 0) {
			// XRAݒ
			g_zi.pi[g_zi.current_player].score -= i;
		}
		g_zi.pi[g_zi.current_player].ri[g_zi.current_round - 1].score_list = g_zi.pi[g_zi.current_player].score;
		SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_zi.current_player, g_zi.pi[g_zi.current_player].score);

		// ϑΏۃEh̐ݒ (80%X^bc)
		if (g_zi.stats_80per == TRUE && g_zi.average_round == 0 && g_zi.pi[g_zi.current_player].score <= g_zi.init_score - (((g_zi.init_score * 8) + 5) / 10)) {
			g_zi.average_round = g_zi.current_round;
			g_zi.average_player = g_zi.current_player;
		}

		if (g_zi.pi[g_zi.current_player].score == 0) {
			// Q[NA
			g_zi.game_flag = FALSE;
			g_zi.pi[g_zi.current_player].darts = g_zi.current_round * THROW_MAX - THROW_MAX;
			for (i = THROW_MAX; i > 0; i--) {
				if (SendMessage(input_wnd, WM_GET_SCORE, i, 0) != 0) {
					g_zi.pi[g_zi.current_player].darts += i;
					break;
				}
			}
			SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
			SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, g_zi.current_player);
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_zi.pi[g_zi.current_player].name);
			SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
			MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_01), MB_ICONINFORMATION);
			SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);
			break;
		}
		// vC[`FW
		g_zi.current_player++;
		if (g_zi.current_player >= g_zi.player_cnt) {
			g_zi.current_player = 0;
			g_zi.current_round++;
		}
		if (g_zi.current_round > g_zi.round_cnt) {
			// SEhI
			g_zi.game_flag = FALSE;
			SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
			for (j = 0, i = 1; i < g_zi.player_cnt; i++) {
				if (g_zi.pi[j].score > g_zi.pi[i].score) {
					j = i;
				}
			}
			for (i = 0; i < g_zi.player_cnt; i++) {
				if (j != i && g_zi.pi[j].score == g_zi.pi[i].score) {
					break;
				}
			}
			if (g_zi.player_cnt > 1 && g_zi.player_cnt > i) {
				// 
				lstrcpy(buf, message_get_res(IDS_STRING_TIE));
			} else {
				SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, j);
				wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_zi.pi[j].name);
			}
			SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
			MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_01), MB_ICONINFORMATION);
			SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);
			break;
		}

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_zi.pi[g_zi.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_zi.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_zi.pi[g_zi.current_player].ri[g_zi.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_DRAW_SCORE:
		*buf = TEXT('\0');
		if (SendMessage(input_wnd, WM_GET_SCORE, 0, 0) == 0) {
			get_arrange(g_zi.pi[g_zi.current_player].score, buf);
		}

		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.bottom -= input_rect.bottom;
		if (g_zi.game_flag == FALSE) {
			result_draw(&bf, &client_rect, &g_zi);
		} else {
			score_draw(&bf, &client_rect, &g_zi, wParam, buf);
		}
		InvalidateRect(hWnd, &client_rect, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SHOW_SCORE:
		if (wParam != -1) {
			g_zi.list_index = wParam;
		} else {
			g_zi.list_index = (g_zi.game_flag == FALSE) ? 0 : g_zi.current_player;
		}
		show_score_01_list(hInst, hWnd, &g_zi);
		break;

	case WM_SAVE_SCORE:
		score_save(hWnd, &g_zi);
		break;

	case WM_SET_SCORE:
	case WM_SET_INPUT_ENABLE:
		return SendMessage(input_wnd, msg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case ID_ACCEL_RETURN:
		case IDC_BUTTON_NEXT:
			SendMessage(input_wnd, WM_COMMAND, wParam, lParam);
			break;
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_01_regist - EBhENX̓o^
 */
BOOL score_01_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = (WNDPROC)score_01_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_01_create - EBhE̍쐬
 */
HWND score_01_create(const HINSTANCE hInstance, const HWND pWnd, int id, const int height)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_TABSTOP,
		0, 0, SCORE_WIDTH, height, pWnd, (HMENU)id, hInstance, NULL);
	return hWnd;
}
/* End of source */
