/*
 * nScore
 *
 * score_cricket.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <stdio.h>

#include "General.h"
#include "Memory.h"
#include "string.h"
#include "Message.h"
#include "file.h"
#include "font.h"
#include "game.h"
#include "score.h"
#include "board.h"
#include "score_cricket.h"
#include "score_cricket_list.h"
#include "score_cricket_option.h"
#include "score_input.h"
#include "robo_cricket.h"
#include "robo_cricket_pattern.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("score_cricket_wnd")
#define ERROR_TITLE					TEXT("nScore - Error")

#define WM_DRAW_SCORE				(WM_APP + 400)

#define OVER_KILL_SCORE				200

#define ID_TIMER_ROBO				1
#ifdef _DEBUG
#define TIMER_INTERVAL_ROBO			0
#else
#define TIMER_INTERVAL_ROBO			1000
#endif

/* Global Variables */
CRICKET_INFO g_cri;
CRICKET_ROBO_INFO cr_ri[ROBO_CNT];
int cr_ri_cnt;
static HWND input_wnd;
static HICON hIcon_mark_1;
static HICON hIcon_mark_2;
static HICON hIcon_mark_3;

extern HINSTANCE hInst;
extern COLOR_INFO ci;
extern BYTE char_set;

typedef struct _BUFFER {
	HBITMAP hbmp;

	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;
	HBRUSH draw_brush;
} BUFFER;

static BUFFER bf;

/* Local Function Prototypes */
static int get_score_string_length(const CRICKET_INFO *cri);
static BOOL get_score_string(const CRICKET_INFO *cri, TCHAR *ret);
static BOOL score_save(const HWND hWnd, const CRICKET_INFO *cri);
static int set_mark(const CRICKET_INFO *cri, const TCHAR *throw_str, int *mark, int *score);
static void set_spoiler(CRICKET_INFO *cri);
static int check_clear(const CRICKET_INFO *cri);
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const CRICKET_INFO *cri);
static BOOL mark_draw(const BUFFER *draw_bf, const RECT *rect, CRICKET_INFO *cri, int *tmp_mark);
static BOOL result_draw(const BUFFER *draw_bf, const RECT *rect, const CRICKET_INFO *cri);
static LRESULT CALLBACK score_cricket_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_cricket_average - ώ擾
 */
float get_cricket_average(const CRICKET_INFO *cri, const int player, float *ret1, float *ret2)
{
	float avg, avg1, avg2;
	int cnt;
	int i;

	cnt = cri->current_round;
	if ((cri->current_player <= player && (cri->game_flag == TRUE || cri->current_player != player)) ||
		(cnt > cri->round_cnt && player == 0)) {
		cnt--;
	}
	avg = avg1 = avg2 = 0;
	for (i = 0; i < cnt; i++) {
		if (cri->pi[player].ri[i].mark_cnt == -1) {
			break;
		}
		if (cri->average_round > 0 && ((cri->average_round == i + 1 && cri->average_player < player) || cri->average_round < i + 1)) {
			break;
		}
		avg += cri->pi[player].ri[i].mark_cnt;
		if (i / 2 == (i + 1) / 2) {
			avg1 += cri->pi[player].ri[i].mark_cnt;
		} else {
			avg2 += cri->pi[player].ri[i].mark_cnt;
		}
	}
	if (i > 0) {
		avg = (float)avg / (float)i;
	} else {
		avg = 0;
	}
	if ((float)((int)((i + 1) / 2)) > 0) {
		avg1 = (float)avg1 / (float)((int)((i + 1) / 2));
	} else {
		avg1 = 0;
	}
	if ((float)((int)(i / 2)) > 0) {
		avg2 = (float)avg2 / (float)((int)(i / 2));
	} else {
		avg2 = 0;
	}

	if (ret1 != NULL) {
		*ret1 = avg1;
	}
	if (ret2 != NULL) {
		*ret2 = avg2;
	}
	return avg;
}

/*
 * get_score_string_length - XRA̒擾
 */
static int get_score_string_length(const CRICKET_INFO *cri)
{
	TCHAR buf[BUF_SIZE];
	TCHAR name[BUF_SIZE];
	int i, j, t;
	int ret = 0;

	// ^Cv
	if (cri->hidden == TRUE) {
		lstrcpy(buf, message_get_res(IDS_STRING_HIDDEN));
	} else if (cri->random == TRUE) {
		lstrcpy(buf, message_get_res(IDS_STRING_RANDOM));
	} else {
		lstrcpy(buf, message_get_res(IDS_STRING_STANDARD));
	}
	if (cri->cut_throat == TRUE && cri->player_cnt > 2) {
		lstrcat(buf, message_get_res(IDS_STRING_OUT_CUT_THROAT));
	}
	ret += lstrlen(buf);
	// O
	for (i = 0; i < cri->player_cnt; i++) {
		if (cri->pi[i].robo == FALSE) {
			lstrcpy(name, cri->pi[i].name);
		} else {
			wsprintf(name, message_get_res(IDS_STRING_ROBO_NAME), i + 1);
		}
		_stprintf(buf, TEXT(",%s,%.2f,,,"), name, get_cricket_average(cri, i, NULL, NULL));
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// ^Cg
	ret += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < cri->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			ret += lstrlen(buf);
		}
		ret++;		//,
		ret += lstrlen(message_get_res(IDS_STRING_MARK));
		ret++;		//,
		ret += lstrlen(message_get_res(IDS_STRING_LIST_SCORE));
	}
	ret += 2;	//CRLF
	// XRA
	for (j = 0; j < cri->current_round; j++) {
		if (j == cri->current_round - 1 && cri->current_player == 0 && cri->game_flag == TRUE) {
			break;
		}
		if (j >= cri->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		ret += lstrlen(buf);
		for (i = 0; i < cri->player_cnt; i++) {
			for (t = 0; t < THROW_MAX; t++) {
				ret++;		//,
				ret += lstrlen(cri->pi[i].ri[j].throw_list[t]);
			}
			if (j == cri->current_round - 1 &&
				(i > cri->current_player || (cri->game_flag == TRUE && i == cri->current_player))) {
				ret++;		//,
				ret++;		//,
				continue;
			}
			if (cri->pi[i].ri[j].score_list == -1) {
				lstrcpy(buf, TEXT(",,"));
			} else if (cri->pi[i].ri[j].mark_cnt == -1) {
				wsprintf(buf, TEXT(",,%d"), cri->pi[i].ri[j].score_list);
			} else {
				wsprintf(buf, TEXT(",%d,%d"), cri->pi[i].ri[j].mark_cnt, cri->pi[i].ri[j].score_list);
			}
			ret += lstrlen(buf);
		}
		ret += 2;	//CRLF
	}
	return ret;
}

/*
 * get_score_string - XRA̍쐬
 */
static BOOL get_score_string(const CRICKET_INFO *cri, TCHAR *ret)
{
	TCHAR *p = ret;
	TCHAR buf[BUF_SIZE];
	TCHAR name[BUF_SIZE];
	int i, j, t;

	// ^Cv
	if (cri->hidden == TRUE) {
		lstrcpy(buf, message_get_res(IDS_STRING_HIDDEN));
	} else if (cri->random == TRUE) {
		lstrcpy(buf, message_get_res(IDS_STRING_RANDOM));
	} else {
		lstrcpy(buf, message_get_res(IDS_STRING_STANDARD));
	}
	if (cri->cut_throat == TRUE && cri->player_cnt > 2) {
		lstrcat(buf, message_get_res(IDS_STRING_OUT_CUT_THROAT));
	}
	lstrcpy(p, buf);
	p += lstrlen(buf);
	// O
	for (i = 0; i < cri->player_cnt; i++) {
		if (cri->pi[i].robo == FALSE) {
			lstrcpy(name, cri->pi[i].name);
		} else {
			wsprintf(name, message_get_res(IDS_STRING_ROBO_NAME), i + 1);
		}
		_stprintf(buf, TEXT(",%s,%.2f,,,"), name, get_cricket_average(cri, i, NULL, NULL));
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// ^Cg
	lstrcpy(p, message_get_res(IDS_STRING_LIST_ROUND));
	p += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < cri->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		*(p++) = TEXT(',');
		lstrcpy(p, message_get_res(IDS_STRING_MARK));
		p += lstrlen(message_get_res(IDS_STRING_MARK));
		*(p++) = TEXT(',');
		lstrcpy(p, message_get_res(IDS_STRING_LIST_SCORE));
		p += lstrlen(message_get_res(IDS_STRING_LIST_SCORE));
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// XRA
	for (j = 0; j < cri->current_round; j++) {
		if (j == cri->current_round - 1 && cri->current_player == 0 && cri->game_flag == TRUE) {
			break;
		}
		if (j >= cri->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		lstrcpy(p, buf);
		p += lstrlen(buf);
		for (i = 0; i < cri->player_cnt; i++) {
			for (t = 0; t < THROW_MAX; t++) {
				*(p++) = TEXT(',');
				lstrcpy(p, cri->pi[i].ri[j].throw_list[t]);
				p += lstrlen(cri->pi[i].ri[j].throw_list[t]);
			}
			if (j == cri->current_round - 1 &&
				(i > cri->current_player || (cri->game_flag == TRUE && i == cri->current_player))) {
				*(p++) = TEXT(',');
				*(p++) = TEXT(',');
				continue;
			}
			if (cri->pi[i].ri[j].score_list == -1) {
				lstrcpy(buf, TEXT(",,"));
			} else if (cri->pi[i].ri[j].mark_cnt == -1) {
				wsprintf(buf, TEXT(",,%d"), cri->pi[i].ri[j].score_list);
			} else {
				wsprintf(buf, TEXT(",%d,%d"), cri->pi[i].ri[j].mark_cnt, cri->pi[i].ri[j].score_list);
			}
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		*(p++) = TEXT('\r');
		*(p++) = TEXT('\n');
	}
	*p = TEXT('\0');
	return TRUE;
}

/*
 * score_save - XRA̕ۑ
 */
static BOOL score_save(const HWND hWnd, const CRICKET_INFO *cri)
{
	TCHAR path[MAX_PATH];
	TCHAR *buf;
	TCHAR err_str[BUF_SIZE];
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	DWORD size;

	// t@C̐
	if (GetDateFormat(0, 0, NULL, TEXT("yyyyMMdd"), date, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	if (GetTimeFormat(0, 0, NULL, TEXT("HHmmss"), time, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	wsprintf(path, TEXT("%s_%s_%s.csv"), message_get_res(IDS_STRING_CRICKET), date, time);

	// t@C̑I
	if (get_save_path(hWnd, message_get_res(IDS_STRING_SAVE_TITLE),
		TEXT("CSV (*.csv)\0*.csv\0All files (*.*)\0*.*\0\0"), TEXT("csv"), path) == FALSE) {
		return FALSE;
	}

	// XRA̍쐬
	size = get_score_string_length(cri);
	buf = mem_alloc(sizeof(TCHAR) * (size + 1));
	if (buf == NULL) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	get_score_string(cri, buf);

	if (file_write_buf(path, (BYTE *)buf, sizeof(TCHAR) * size, err_str) == FALSE) {
		mem_free(&buf);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	mem_free(&buf);
	return TRUE;
}

/*
 * set_mark - }[NƃXRAݒ
 */
static int set_mark(const CRICKET_INFO *cri, const TCHAR *throw_str, int *mark, int *score)
{
	int i, j;
	int cnt;
	int ret = 0;
	int no = -1;
	int sc;

	// ԍƃJEg擾
	if (*throw_str == TEXT('\0')) {
		return 0;
	} if (lstrcmp(throw_str, message_get_res(IDS_STRING_D_BULL)) == 0) {
		i = 0;
		cnt = 2;
	} else if (lstrcmp(throw_str, message_get_res(IDS_STRING_S_BULL)) == 0) {
		i = 0;
		cnt = 1;
	} else {
		switch (*throw_str) {
		case CHAR_TRIPLE:
			cnt = 3;
			break;
		case CHAR_DOUBLE:
			cnt = 2;
			break;
		case CHAR_SINGLE:
			cnt = 1;
			break;
		}
		i = _ttoi(throw_str + 1);
		if (i == 0) {
			i = -1;
		}
	}
	// }[Nݒ
	for (j = 0; j < NUMBER_CNT; j++) {
		if (cri->number[j] == i) {
			mark[j] += cnt;
			ret = cnt;
			no = j;
			break;
		}
	}
	if (no != -1 && mark[no] > MARK_CNT) {
		// ̃vC[̃}[N`FbN
		for (i = 0; i < cri->player_cnt; i++) {
			if (i != cri->current_player && cri->pi[i].spoiler == FALSE && cri->pi[i].mark[no] < MARK_CNT) {
				break;
			}
		}
		if (i >= cri->player_cnt) {
			ret -= mark[no] - MARK_CNT;
		} else {
			// XRAvZ
			sc = cri->number[no];
			if (sc == 0) {
				// bull
				sc = 25;
			}
			sc = (mark[no] - MARK_CNT) * sc;

			// XRAǉ
			if (cri->cut_throat == TRUE && cri->player_cnt > 2) {
				for (j = 0; j < cri->player_cnt; j++) {
					if (j != cri->current_player && cri->pi[j].spoiler == FALSE && cri->pi[j].mark[no] < MARK_CNT) {
						score[j] += sc;
					}
				}
			} else {
				if (cri->player_cnt == 2 && cri->over_kill == TRUE) {
					// Over kill
					if ((cri->current_player == 0 && cri->pi[0].score + score[0] < cri->pi[1].score + OVER_KILL_SCORE) ||
						(cri->current_player == 1 && cri->pi[1].score + score[1] < cri->pi[0].score + OVER_KILL_SCORE)){
						score[cri->current_player] += sc;
					} else {
						ret -= mark[no] - MARK_CNT;
					}
				} else {
					score[cri->current_player] += sc;
				}
			}
		}
		// }[Nݒ
		mark[no] = MARK_CNT;
	}
	return ret;
}

/*
 * set_spoiler - spoilerݒ
 */
static void set_spoiler(CRICKET_INFO *cri)
{
	int i, j;

	for (i = 0; i < cri->player_cnt; i++) {
		cri->pi[i].spoiler = FALSE;
	}
	for (j = 0; j < cri->player_cnt; j++) {
		// S}[NcloseĂ邩`FbN
		for (i = 0; i < NUMBER_CNT; i++) {
			if (cri->pi[j].mark[i] < MARK_CNT) {
				break;
			}
		}
		if (i >= NUMBER_CNT) {
			// _Ⴂꍇspoilerݒ
			for (i = 0; i < cri->player_cnt; i++) {
				if (j != i && cri->pi[j].score <= cri->pi[i].score) {
					cri->pi[i].spoiler = TRUE;
				}
			}
		}
	}
}

/*
 * check_clear - NA
 */
static int check_clear(const CRICKET_INFO *cri)
{
	int i, j;
	BOOL no_mark_flag = FALSE;

	for (j = 0; j < cri->player_cnt; j++) {
		// S}[NcloseĂ邩`FbN
		for (i = 0; i < NUMBER_CNT; i++) {
			if (cri->pi[j].mark[i] < MARK_CNT) {
				no_mark_flag = TRUE;
				break;
			}
		}
		if (i >= NUMBER_CNT) {
			// _ŃgbvɂȂĂ邩`FbN
			for (i = 0; i < cri->player_cnt; i++) {
				if (cri->cut_throat == TRUE && cri->player_cnt > 2) {
					if (j != i && cri->pi[j].score > cri->pi[i].score) {
						break;
					}
				} else {
					if (j != i && cri->pi[j].score < cri->pi[i].score) {
						break;
					}
				}
			}
			if (i >= cri->player_cnt) {
				return j;
			}
		}
	}
	if (cri->player_cnt > 1 && no_mark_flag == FALSE) {
		return -2;
	}
	return -1;
}

/*
 * check_clear_tmp - ro߂ł̃NA
 */
static int check_clear_tmp(const CRICKET_INFO *cri)
{
	int i, j;

	for (j = 0; j < cri->player_cnt; j++) {
		// S}[NcloseĂ邩`FbN
		for (i = 0; i < NUMBER_CNT; i++) {
			if (j == g_cri.current_player) {
				if (cri->tmp_mark[i] < MARK_CNT) {
					break;
				}
			} else {
				if (cri->pi[j].mark[i] < MARK_CNT) {
					break;
				}
			}
		}
		if (i >= NUMBER_CNT) {
			// _ŃgbvɂȂĂ邩`FbN
			for (i = 0; i < cri->player_cnt; i++) {
				if (cri->cut_throat == TRUE && cri->player_cnt > 2) {
					if (j != i && (cri->pi[j].score + cri->tmp_score[j]) > (cri->pi[i].score + cri->tmp_score[i])) {
						break;
					}
				} else {
					if (j != i && (cri->pi[j].score + cri->tmp_score[j]) < (cri->pi[i].score + cri->tmp_score[i])) {
						break;
					}
				}
			}
			if (i >= cri->player_cnt) {
				return j;
			}
		}
	}
	return -1;
}

/*
 * round_draw - Eh̕`
 */
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const CRICKET_INFO *cri)
{
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	int i;

	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	i = (cri->current_round > cri->round_cnt) ? cri->round_cnt : cri->current_round;
	wsprintf(buf, message_get_res(IDS_STRING_DRAW_ROUND), i, cri->round_cnt);
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), draw_rect, 0);
	if (cri->cut_throat == TRUE && cri->player_cnt > 2) {
		DrawText(draw_bf->draw_dc,
			message_get_res(IDS_STRING_CUT_THROAT), lstrlen(message_get_res(IDS_STRING_CUT_THROAT)),
			draw_rect, DT_RIGHT);
	}
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * mark_draw - }[N̕`
 */
static BOOL mark_draw(const BUFFER *draw_bf, const RECT *rect, CRICKET_INFO *cri, int *tmp_mark)
{
	TEXTMETRIC tm;
	RECT draw_rect, mark_rect;
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	TCHAR spoiler_str[BUF_SIZE];
	int *mark;
	int score_width;
	int i, j;

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	score_width = draw_rect.right - draw_rect.left;
	draw_rect.left += 10;
	draw_rect.right -= 10;
	mark_rect = draw_rect;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, cri);

	// io[
	hfont = font_create(FONT_NAME, 24, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top = 35;
	for (i = 0; i < NUMBER_CNT; i++) {
		j = 0;
		if (cri->hidden == TRUE) {
			for (; j < cri->player_cnt; j++) {
				if (j == cri->current_player && tmp_mark != NULL) {
					if (tmp_mark[i] != 0) {
						break;
					}
				} else if (cri->pi[j].mark[i] != 0) {
					break;
				}
			}
		}
		if (cri->hidden == FALSE || j < cri->player_cnt) {
			if (cri->number[i] == 0) {
				lstrcpy(buf, message_get_res(IDS_STRING_BULL));
			} else {
				_itot(cri->number[i], buf, 10);
			}
		} else {
			// hidden
			lstrcpy(buf, message_get_res(IDS_STRING_HIDDEN_NUMBER));
		}
		for (j = 0; j < cri->player_cnt; j++) {
			if (j == cri->current_player && tmp_mark != NULL) {
				if (tmp_mark[i] < MARK_CNT) {
					break;
				}
			} else if (cri->pi[j].spoiler == FALSE && cri->pi[j].mark[i] < MARK_CNT) {
				break;
			}
		}
		if (j >= cri->player_cnt) {
			// ScloseĂ
			SetTextColor(draw_bf->draw_dc, ci.close_text);
		} else {
			SetTextColor(draw_bf->draw_dc, ci.text);
		}
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		draw_rect.top += tm.tmHeight + 5;
	}
	// }[N
	for (j = 0; j < cri->player_cnt; j++) {
		mark_rect.top = 35;
		switch (j + ((cri->player_cnt <= 2) ? 1 : 0)) {
		case 0: mark_rect.left = 60; break;
		case 1: mark_rect.left = 145; break;
		case 2: mark_rect.left = score_width - 145 - 32; break;
		case 3: mark_rect.left = score_width - 60 - 32; break;
		}
		if (cri->pi[j].spoiler == FALSE) {
			if (j == cri->current_player && tmp_mark != NULL) {
				mark = tmp_mark;
			} else {
				mark = cri->pi[j].mark;
			}
			for (i = 0; i < NUMBER_CNT; i++) {
				switch (mark[i]) {
				case 0:
					break;
				case 1:
					DrawIcon(draw_bf->draw_dc, mark_rect.left, mark_rect.top, hIcon_mark_1);
					break;
				case 2:
					DrawIcon(draw_bf->draw_dc, mark_rect.left, mark_rect.top, hIcon_mark_2);
					break;
				case 3:
					DrawIcon(draw_bf->draw_dc, mark_rect.left, mark_rect.top, hIcon_mark_3);
					break;
				}
				mark_rect.top += tm.tmHeight + 5;
			}
		} else {
			// spoiler
			SetTextColor(draw_bf->draw_dc, ci.spoiler_text);
			lstrcpy(spoiler_str, message_get_res(IDS_STRING_SPOILER));
			for (i = 0; i < NUMBER_CNT; i++) {
				*buf = spoiler_str[i];
				*(buf + 1) = TEXT('\0');
				DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &mark_rect, 0);
				mark_rect.top += tm.tmHeight + 5;
			}
		}
	}
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * result_draw - ʂ̕`
 */
static BOOL result_draw(const BUFFER *draw_bf, const RECT *rect, const CRICKET_INFO *cri)
{
	TEXTMETRIC tm;
	RECT draw_rect;
	HFONT hfont, hRetFont;
	HPEN pen, ret_pen;
	TCHAR buf[BUF_SIZE];
	float avg, avg1, avg2;
	int score_width;
	int j;
	int top1, top2;
	int line_top1, line_top2;

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	draw_rect.left += 10;
	draw_rect.right -= 10;
	score_width = draw_rect.right - draw_rect.left;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, cri);

	// vC[
	for (j = 0; j < cri->player_cnt; j++) {
		draw_rect.top = 35;
		draw_rect.left = (j + ((cri->player_cnt <= 2) ? 1 : 0)) * (score_width / 4) + 20;
		draw_rect.right = draw_rect.left + 100;

		// ώ擾
		avg = get_cricket_average(cri, j, &avg1, &avg2);

		// Average title
		hfont = font_create(FONT_NAME, 12, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_DRAW_AVERAGE), lstrlen(message_get_res(IDS_STRING_DRAW_AVERAGE)), &draw_rect, DT_CENTER);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		draw_rect.top += tm.tmHeight + 5;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);
		// Average
		hfont = font_create(FONT_NAME, 20, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		_stprintf(buf, TEXT("%.2f"), avg);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		line_top1 = draw_rect.top + tm.tmHeight + 15;
		draw_rect.top += tm.tmHeight + 30;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);

		// First title & Second title
		hfont = font_create(FONT_NAME, 12, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_FIRST), lstrlen(message_get_res(IDS_STRING_FIRST)), &draw_rect, DT_CENTER);
		top1 = draw_rect.top + tm.tmHeight + 5;
		draw_rect.top += tm.tmHeight + 50;
		DrawText(draw_bf->draw_dc, message_get_res(IDS_STRING_SECOND), lstrlen(message_get_res(IDS_STRING_SECOND)), &draw_rect, DT_CENTER);
		top2 = draw_rect.top + tm.tmHeight + 5;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);
		// First Average & Second Average
		hfont = font_create(FONT_NAME, 14, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		GetTextMetrics(draw_bf->draw_dc, &tm);
		draw_rect.top = top1;
		_stprintf(buf, TEXT("%.2f"), avg1);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		draw_rect.top = top2;
		_stprintf(buf, TEXT("%.2f"), avg2);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		line_top2 = draw_rect.top + tm.tmHeight + 15;
		draw_rect.top += tm.tmHeight + 30;
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);

		if (cri->pi[j].robo == TRUE) {
			// ROBO info
			hfont = font_create(FONT_NAME, 12, char_set, 0, FALSE, FALSE);
			hRetFont = SelectObject(draw_bf->draw_dc, hfont);
			GetTextMetrics(draw_bf->draw_dc, &tm);
			DrawText(draw_bf->draw_dc, cr_ri[cri->pi[j].robo_index].name, lstrlen(cr_ri[cri->pi[j].robo_index].name), &draw_rect, DT_CENTER);
			draw_rect.top += tm.tmHeight + 5;
			wsprintf(buf, TEXT("Lv.%d"), cri->pi[j].robo_lv + 1);
			DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
			SelectObject(draw_bf->draw_dc, hRetFont);
			DeleteObject(hfont);
		}
	}
	// ̕`
	pen = CreatePen(PS_SOLID, 0, ci.line);
	ret_pen = SelectObject(draw_bf->draw_dc, pen);
	MoveToEx(draw_bf->draw_dc, rect->left + 15, line_top1, NULL);
	LineTo(draw_bf->draw_dc, rect->right - 15, line_top1);
	MoveToEx(draw_bf->draw_dc, rect->left + 15, line_top2, NULL);
	LineTo(draw_bf->draw_dc, rect->right - 15, line_top2);
	SelectObject(draw_bf->draw_dc, ret_pen);
	DeleteObject(pen);
	return TRUE;
}

/*
 * score_cricket_proc - EBhEvV[W
 */
static LRESULT CALLBACK score_cricket_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	static int throw_cnt;
	static BOOL show_avg;
	HDC hdc;
	PAINTSTRUCT ps;
	RECT client_rect, input_rect;
	TCHAR buf[BUF_SIZE];
	TCHAR tmp[BUF_SIZE];
	int i, j;

	switch (msg) {
	case WM_CREATE:
		// {̏
		for (cr_ri_cnt = 0; cr_ri_cnt < 10; cr_ri_cnt++) {
			if (get_robo_info(&(cr_ri[cr_ri_cnt]), cr_ri_cnt) == FALSE) {
				break;
			}
		}

		// ̓EBhȄ
		input_wnd = score_input_create(hInst, hWnd);
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.right = input_rect.right;
		client_rect.bottom += input_rect.bottom;
		SetWindowPos(hWnd, 0, 0, 0, client_rect.right, client_rect.bottom, SWP_NOMOVE | SWP_NOZORDER);
		SetWindowPos(input_wnd, 0, 0, client_rect.bottom - input_rect.bottom, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER);

		// `p
		hdc = GetDC(hWnd);
		client_rect.bottom -= input_rect.bottom;
		bf.draw_dc = CreateCompatibleDC(hdc);
		bf.draw_bmp = CreateCompatibleBitmap(hdc, client_rect.right, client_rect.bottom);
		bf.draw_ret_bmp = SelectObject(bf.draw_dc, bf.draw_bmp);
		ReleaseDC(hWnd, hdc);
		bf.draw_brush = CreateSolidBrush(ci.background);
		FillRect(bf.draw_dc, &client_rect, bf.draw_brush);

		hIcon_mark_1 = (HICON)LoadImage(hInst, MAKEINTRESOURCE(IDI_ICON_MARK_1), IMAGE_ICON, 32, 32, 0);
		hIcon_mark_2 = (HICON)LoadImage(hInst, MAKEINTRESOURCE(IDI_ICON_MARK_2), IMAGE_ICON, 32, 32, 0);
		hIcon_mark_3 = (HICON)LoadImage(hInst, MAKEINTRESOURCE(IDI_ICON_MARK_3), IMAGE_ICON, 32, 32, 0);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// EBhE̔j
		SendMessage(input_wnd, WM_CLOSE, 0, 0);

		SelectObject(bf.draw_dc, bf.draw_ret_bmp);
		DeleteObject(bf.draw_bmp);
		DeleteDC(bf.draw_dc);
		DeleteObject(bf.draw_brush);

		DestroyIcon(hIcon_mark_1);
		DestroyIcon(hIcon_mark_2);
		DestroyIcon(hIcon_mark_3);
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		SetFocus(input_wnd);
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf.draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_LBUTTONDBLCLK:
		if (g_cri.game_flag == FALSE) {
			show_avg = !show_avg;
			SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);
		}
		break;

	case WM_GAME_OPTION:
		// NPbgݒ
		return set_option_cricket(hInst, hWnd, &g_cri);

	case WM_GAME_INIT:
		// Q[̏
		show_avg = FALSE;
		g_cri.game_flag = TRUE;
		if (g_cri.player_cnt <= 2) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, 0, (LPARAM)TEXT(""));
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, 0, (LPARAM)TEXT(""));
		}
		for (i = 0; i < g_cri.player_cnt; i++) {
			if (g_cri.pi[i].robo == FALSE) {
				SendMessage(input_wnd, WM_SET_PLAYER_NAME, i + ((g_cri.player_cnt <= 2) ? 1 : 0), (LPARAM)g_cri.pi[i].name);
			} else {
				wsprintf(buf, message_get_res(IDS_STRING_ROBO_NAME), i + 1);
				SendMessage(input_wnd, WM_SET_PLAYER_NAME, i + ((g_cri.player_cnt <= 2) ? 1 : 0), (LPARAM)buf);
			}
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i + ((g_cri.player_cnt <= 2) ? 1 : 0), g_cri.pi[i].score);
		}
		for (; i < PLAYER_MAX; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i + ((g_cri.player_cnt <= 2) ? 1 : 0), (LPARAM)TEXT(""));
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, i + ((g_cri.player_cnt <= 2) ? 1 : 0), (LPARAM)TEXT(""));
		}
		g_cri.current_round = 1;
		g_cri.current_player = 0;
		if (g_cri.pi[g_cri.current_player].robo == FALSE) {
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_cri.pi[g_cri.current_player].name);
		} else {
			wsprintf(tmp, message_get_res(IDS_STRING_ROBO_NAME), g_cri.current_player + 1);
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), tmp);
		}
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(GetParent(hWnd), WM_SET_BULL_TYPE, FALSE, 0);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_cri.current_player + ((g_cri.player_cnt <= 2) ? 1 : 0));
		SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, -1);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);

		if (g_cri.round_cnt > 0 && g_cri.pi[g_cri.current_player].robo == TRUE) {
			// {
			EnableWindow(hWnd, FALSE);
			SetFocus(NULL);
			SendMessage(GetParent(hWnd), WM_SET_BOARD_ENABLE, FALSE, 0);
			throw_cnt = 0;
			SetTimer(hWnd, ID_TIMER_ROBO, TIMER_INTERVAL_ROBO, NULL);
		}
		break;

	case WM_GAME_STOP:
		KillTimer(hWnd, ID_TIMER_ROBO);
		EnableWindow(hWnd, TRUE);
		g_cri.game_flag = FALSE;
		break;

	case WM_CHANGE_FOCUS:
		if (g_cri.game_flag == FALSE) {
			break;
		}
		ZeroMemory(g_cri.tmp_mark, sizeof(int) * NUMBER_CNT);
		ZeroMemory(g_cri.tmp_score, sizeof(int) * PLAYER_MAX);
		for (i = 0; i < NUMBER_CNT; i++) {
			g_cri.tmp_mark[i] = g_cri.pi[g_cri.current_player].mark[i];
		}
		// }[Nݒ
		SendMessage(input_wnd, WM_GET_SCORE_STR, 1, (LPARAM)buf);
		set_mark(&g_cri, buf, g_cri.tmp_mark, g_cri.tmp_score);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 2, (LPARAM)buf);
		set_mark(&g_cri, buf, g_cri.tmp_mark, g_cri.tmp_score);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 3, (LPARAM)buf);
		set_mark(&g_cri, buf, g_cri.tmp_mark, g_cri.tmp_score);

		// \XV
		for (i = 0; i < g_cri.player_cnt; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i + ((g_cri.player_cnt <= 2) ? 1 : 0),
				g_cri.pi[i].score + g_cri.tmp_score[i]);
		}
		SendMessage(hWnd, WM_DRAW_SCORE, 0, (LPARAM)g_cri.tmp_mark);

		// I
		if (check_clear_tmp(&g_cri) != -1) {
			SendMessage(input_wnd, WM_SET_FOCUS_NEXT_BTN, 0, 0);
		}
		break;

	case WM_PREV_PLAYER:
		if (g_cri.current_player <= 0 && g_cri.current_round <= 1) {
			break;
		}
		if (g_cri.pi[g_cri.current_player].robo == TRUE &&
			IsWindowEnabled(GetDlgItem(input_wnd, IDC_EDIT_1)) != FALSE) {
			break;
		}
		SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_cri.current_player + ((g_cri.player_cnt <= 2) ? 1 : 0),
			g_cri.pi[g_cri.current_player].score);

		do {
			g_cri.current_player--;
			if (g_cri.current_player < 0) {
				g_cri.current_player = g_cri.player_cnt - 1;
				g_cri.current_round--;
				if (g_cri.current_round < 1) {
					g_cri.current_round = 1;
					break;
				}
			}
		} while (g_cri.pi[g_cri.current_player].spoiler == TRUE);

		for (i = 0; i < g_cri.player_cnt; i++) {
			g_cri.pi[i].score = g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].prev_score_list[i];
		}
		CopyMemory(g_cri.pi[g_cri.current_player].mark,
			g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].mark_list, sizeof(int) * NUMBER_CNT);

		// ϑΏۃEh̏
		if (g_cri.average_round >= g_cri.current_round && g_cri.average_player >= g_cri.current_player) {
			g_cri.average_round = 0;
			g_cri.average_player = 0;
		}

		// spoilerݒ
		if (g_cri.cut_throat == TRUE && g_cri.player_cnt > 2) {
			set_spoiler(&g_cri);
		}

		// 
		if (g_cri.pi[g_cri.current_player].robo == FALSE) {
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_cri.pi[g_cri.current_player].name);
		} else {
			wsprintf(tmp, message_get_res(IDS_STRING_ROBO_NAME), g_cri.current_player + 1);
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), tmp);
		}
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_cri.current_player + ((g_cri.player_cnt <= 2) ? 1 : 0));
		SendMessage(input_wnd, WM_SET_SCORE_ENABLE, (g_cri.pi[g_cri.current_player].robo == TRUE) ? FALSE : TRUE, 0);
		SendMessage(GetParent(hWnd), WM_SET_BOARD_ENABLE, (g_cri.pi[g_cri.current_player].robo == TRUE) ? FALSE : TRUE, 0);
		KillTimer(hWnd, ID_TIMER_ROBO);
		EnableWindow(hWnd, TRUE);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_NEXT_PLAYER:
		EnableWindow(hWnd, TRUE);
		SendMessage(GetParent(hWnd), WM_SET_BOARD_ENABLE, TRUE, 0);

		// XRA擾
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_GET_SCORE_FULL_STR, i + 1, (LPARAM)g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].throw_list[i]);
		}
		// }[Nݒ
		ZeroMemory(g_cri.tmp_score, sizeof(int) * PLAYER_MAX);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 1, (LPARAM)buf);
		i = set_mark(&g_cri, buf, g_cri.pi[g_cri.current_player].mark, g_cri.tmp_score);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 2, (LPARAM)buf);
		i += set_mark(&g_cri, buf, g_cri.pi[g_cri.current_player].mark, g_cri.tmp_score);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 3, (LPARAM)buf);
		i += set_mark(&g_cri, buf, g_cri.pi[g_cri.current_player].mark, g_cri.tmp_score);

		// }[Nݒ
		g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].mark_cnt = i;
		// ϑΏۃEh̐ݒ (80%X^bc)
		if (g_cri.stats_80per == TRUE && g_cri.average_round == 0) {
			j = 0;
			for (i = 0; i < NUMBER_CNT; i++) {
				if (g_cri.pi[g_cri.current_player].mark[i] >= MARK_CNT) {
					j++;
				}
			}
			if (j >= NUMBER_CNT - 1) {
				g_cri.average_round = g_cri.current_round;
				g_cri.average_player = g_cri.current_player;
			}
		}

		// XRAݒ
		for (i = 0; i < g_cri.player_cnt; i++) {
			g_cri.pi[i].score += g_cri.tmp_score[i];
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i + ((g_cri.player_cnt <= 2) ? 1 : 0), g_cri.pi[i].score);
		}
		g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].score_list = g_cri.pi[g_cri.current_player].score;

		// NA
		j = check_clear(&g_cri);
		if (j != -1) {
			// Q[NA
			show_avg = TRUE;
			g_cri.game_flag = FALSE;
			SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
			if (j == -2) {
				// 
				lstrcpy(buf, message_get_res(IDS_STRING_TIE));
			} else {
				SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, j + ((g_cri.player_cnt <= 2) ? 1 : 0));
				if (g_cri.pi[j].robo == FALSE) {
					wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_cri.pi[j].name);
				} else {
					wsprintf(tmp, message_get_res(IDS_STRING_ROBO_NAME), j + 1);
					wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), tmp);
				}
			}
			SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
			MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_CRICKET), MB_ICONINFORMATION);
			SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);
			break;
		}

		// spoilerݒ
		if (g_cri.cut_throat == TRUE && g_cri.player_cnt > 2) {
			set_spoiler(&g_cri);
		}

		// vC[`FW
		do {
			g_cri.current_player++;
			if (g_cri.current_player >= g_cri.player_cnt) {
				g_cri.current_player = 0;
				g_cri.current_round++;
				if (g_cri.current_round > g_cri.round_cnt) {
					break;
				}
			}
			if (g_cri.pi[g_cri.current_player].spoiler == TRUE) {
				g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].mark_cnt = -1;
				g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].score_list = -1;
			}
		} while (g_cri.pi[g_cri.current_player].spoiler == TRUE);

		if (g_cri.current_round > g_cri.round_cnt) {
			// SEhI
			show_avg = TRUE;
			g_cri.game_flag = FALSE;
			SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
			for (j = 0, i = 1; i < g_cri.player_cnt; i++) {
				if (g_cri.cut_throat == TRUE && g_cri.player_cnt > 2) {
					if (g_cri.pi[j].score > g_cri.pi[i].score) {
						j = i;
					}
				} else {
					if (g_cri.pi[j].score < g_cri.pi[i].score) {
						j = i;
					}
				}
			}
			for (i = 0; i < g_cri.player_cnt; i++) {
				if (j != i && g_cri.pi[j].score == g_cri.pi[i].score) {
					break;
				}
			}
			if (g_cri.player_cnt > 1 && g_cri.player_cnt > i) {
				// 
				lstrcpy(buf, message_get_res(IDS_STRING_TIE));
			} else {
				SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, j + ((g_cri.player_cnt <= 2) ? 1 : 0));
				if (g_cri.pi[j].robo == FALSE) {
					wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_cri.pi[j].name);
				} else {
					wsprintf(tmp, message_get_res(IDS_STRING_ROBO_NAME), j + 1);
					wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), tmp);
				}
			}
			SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
			MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_CRICKET), MB_ICONINFORMATION);
			SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);
			break;
		}

		// ߂pɑޔ
		for (i = 0; i < g_cri.player_cnt; i++) {
			g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].prev_score_list[i] = g_cri.pi[i].score;
		}
		CopyMemory(g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].mark_list,
			g_cri.pi[g_cri.current_player].mark, sizeof(int) * NUMBER_CNT);

		// 
		if (g_cri.pi[g_cri.current_player].robo == FALSE) {
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_cri.pi[g_cri.current_player].name);
		} else {
			wsprintf(tmp, message_get_res(IDS_STRING_ROBO_NAME), g_cri.current_player + 1);
			wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), tmp);
		}
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		for (i = 0; i < NUMBER_CNT; i++) {
			g_cri.tmp_mark[i] = g_cri.pi[g_cri.current_player].mark[i];
		}
		ZeroMemory(g_cri.tmp_score, sizeof(int) * PLAYER_MAX);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_cri.current_player + ((g_cri.player_cnt <= 2) ? 1 : 0));
		SendMessage(input_wnd, WM_SET_SCORE_ENABLE, TRUE, 0);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);

		if (g_cri.pi[g_cri.current_player].robo == TRUE) {
			// {
			SendMessage(GetParent(hWnd), WM_SET_BOARD_ENABLE, FALSE, 0);
			if (*g_cri.pi[g_cri.current_player].ri[g_cri.current_round - 1].throw_list[0] != TEXT('\0')) {
				SendMessage(input_wnd, WM_SET_SCORE_ENABLE, FALSE, 0);
			} else {
				EnableWindow(hWnd, FALSE);
				SetFocus(NULL);
				throw_cnt = 0;
				SetTimer(hWnd, ID_TIMER_ROBO, TIMER_INTERVAL_ROBO, NULL);
			}
		}
		break;

	case WM_TIMER:
		switch (wParam) {
		case ID_TIMER_ROBO:
			if (throw_cnt >= THROW_MAX) {
				KillTimer(hWnd, ID_TIMER_ROBO);
				SendMessage(hWnd, WM_NEXT_PLAYER, 0, 0);
				break;
			}
			i = cr_ri[g_cri.pi[g_cri.current_player].robo_index].func(hWnd, &g_cri, g_cri.current_player, throw_cnt + 1);
			robo_get_throw(g_cri.pi[g_cri.current_player].robo_lv + 1, i, buf);
			SendMessage(input_wnd, WM_SET_SCORE, throw_cnt + 1, (LPARAM)buf);
			SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
			throw_cnt++;

			// I
			if (check_clear_tmp(&g_cri) != -1) {
				throw_cnt = THROW_MAX;
			}
			break;
		}
		break;

	case WM_DRAW_SCORE:
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.bottom -= input_rect.bottom;
		if (show_avg != FALSE) {
			result_draw(&bf, &client_rect, &g_cri);
		} else {
			mark_draw(&bf, &client_rect, &g_cri, (int *)lParam);
		}
		InvalidateRect(hWnd, &client_rect, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SHOW_SCORE:
		if (wParam != -1) {
			g_cri.list_index = wParam - ((g_cri.player_cnt <= 2) ? 1 : 0);
		} else {
			g_cri.list_index = (g_cri.game_flag == FALSE) ? 0 : g_cri.current_player;
		}
		show_score_cricket_list(hInst, hWnd, &g_cri);
		break;

	case WM_SAVE_SCORE:
		score_save(hWnd, &g_cri);
		break;

	case WM_SET_SCORE:
	case WM_SET_INPUT_ENABLE:
		return SendMessage(input_wnd, msg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case IDC_BUTTON_NEXT:
			if (IsWindowEnabled(hWnd) != FALSE) {
				SendMessage(input_wnd, WM_COMMAND, wParam, lParam);
			}
			break;

		case ID_ACCEL_RETURN:
			if (g_cri.game_flag == FALSE) {
				show_avg = !show_avg;
				SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);
			} else if (IsWindowEnabled(hWnd) != FALSE) {
				SendMessage(input_wnd, WM_COMMAND, wParam, lParam);
			}
			break;
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_cricket_regist - EBhENX̓o^
 */
BOOL score_cricket_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = (WNDPROC)score_cricket_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_cricket_create - EBhE̍쐬
 */
HWND score_cricket_create(const HINSTANCE hInstance, const HWND pWnd, int id, const int height)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_TABSTOP,
		0, 0, SCORE_WIDTH, height, pWnd, (HMENU)id, hInstance, NULL);
	return hWnd;
}
/* End of source */
