/*
 * nScore
 *
 * score_cricket_countup.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <stdio.h>

#include "General.h"
#include "Memory.h"
#include "string.h"
#include "Message.h"
#include "file.h"
#include "font.h"
#include "game.h"
#include "board.h"
#include "score.h"
#include "score_cricket_countup.h"
#include "score_cricket_countup_list.h"
#include "score_cricket_countup_option.h"
#include "score_input.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("score_cricket_countup_wnd")
#define ERROR_TITLE					TEXT("nScore - Error")

#define WM_DRAW_SCORE				(WM_APP + 400)

/* Global Variables */
CRICKET_COUNTUP_INFO g_ccui;
static HWND input_wnd;

extern HINSTANCE hInst;
extern COLOR_INFO ci;
extern BYTE char_set;

typedef struct _BUFFER {
	HBITMAP hbmp;

	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;
	HBRUSH draw_brush;
} BUFFER;

static BUFFER bf;

/* Local Function Prototypes */
static int get_score_string_length(const CRICKET_COUNTUP_INFO *ccui);
static BOOL get_score_string(const CRICKET_COUNTUP_INFO *ccui, TCHAR *ret);
static BOOL score_save(const HWND hWnd, const CRICKET_COUNTUP_INFO *ccui);
static int get_score(CRICKET_COUNTUP_INFO *ccui, const TCHAR *throw_str, CRICKET_COUNTUP_ROUND_INFO *ri);
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const CRICKET_COUNTUP_INFO *ccui);
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const CRICKET_COUNTUP_INFO *ccui, const int score);
static BOOL result_draw(const BUFFER *draw_bf, const RECT *rect, const CRICKET_COUNTUP_INFO *ccui);
static LRESULT CALLBACK score_cricket_countup_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_score_string_length - XRA̒擾
 */
static int get_score_string_length(const CRICKET_COUNTUP_INFO *ccui)
{
	TCHAR buf[BUF_SIZE];
	int i, j, t;
	int ret = 0;

	ret += lstrlen(message_get_res(IDS_STRING_CRICKET_COUNTUP));
	// O
	for (i = 0; i < ccui->player_cnt; i++) {
		wsprintf(buf, TEXT(",%s,,,,"), ccui->pi[i].name);
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// ^Cg
	ret += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < ccui->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			ret += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_MARK));
		ret += lstrlen(buf);
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		ret += lstrlen(buf);
	}
	ret += 2;	//CRLF
	// XRA
	for (j = 0; j < ccui->current_round; j++) {
		if (j == ccui->current_round - 1 && ccui->current_player == 0 && ccui->game_flag == TRUE) {
			break;
		}
		if (j >= ccui->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		ret += lstrlen(buf);
		for (i = 0; i < ccui->player_cnt; i++) {
			// throw
			for (t = 0; t < THROW_MAX; t++) {
				ret++;		//,
				ret += lstrlen(ccui->pi[i].ri[j].throw_list[t]);
			}
			if (j == ccui->current_round - 1 &&
				(i > ccui->current_player || (ccui->game_flag == TRUE && i == ccui->current_player))) {
				ret++;		//,
				ret++;		//,
				continue;
			}
			// Mark
			wsprintf(buf, TEXT(",%d"), ccui->pi[i].ri[j].mark_list);
			ret += lstrlen(buf);
			// Score
			wsprintf(buf, TEXT(",%d"), ccui->pi[i].ri[j].score_list);
			ret += lstrlen(buf);
		}
		ret += 2;	//CRLF
	}
	return ret;
}

/*
 * get_score_string - XRA̍쐬
 */
static BOOL get_score_string(const CRICKET_COUNTUP_INFO *ccui, TCHAR *ret)
{
	TCHAR *p = ret;
	TCHAR buf[BUF_SIZE];
	int i, j, t;

	lstrcpy(p, message_get_res(IDS_STRING_CRICKET_COUNTUP));
	p += lstrlen(message_get_res(IDS_STRING_CRICKET_COUNTUP));
	// O
	for (i = 0; i < ccui->player_cnt; i++) {
		wsprintf(buf, TEXT(",%s,,,,"), ccui->pi[i].name);
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// ^Cg
	lstrcpy(p, message_get_res(IDS_STRING_LIST_ROUND));
	p += lstrlen(message_get_res(IDS_STRING_LIST_ROUND));
	for (i = 0; i < ccui->player_cnt; i++) {
		for (t = 0; t < THROW_MAX; t++) {
			wsprintf(buf, TEXT(",%d"), t + 1);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_MARK));
		lstrcpy(p, buf);
		p += lstrlen(buf);
		wsprintf(buf, TEXT(",%s"), message_get_res(IDS_STRING_LIST_SCORE));
		lstrcpy(p, buf);
		p += lstrlen(buf);
	}
	*(p++) = TEXT('\r');
	*(p++) = TEXT('\n');
	// XRA
	for (j = 0; j < ccui->current_round; j++) {
		if (j == ccui->current_round - 1 && ccui->current_player == 0 && ccui->game_flag == TRUE) {
			break;
		}
		if (j >= ccui->round_cnt) {
			break;
		}
		wsprintf(buf, TEXT("%d"), j + 1);
		lstrcpy(p, buf);
		p += lstrlen(buf);
		for (i = 0; i < ccui->player_cnt; i++) {
			// throw
			for (t = 0; t < THROW_MAX; t++) {
				*(p++) = TEXT(',');
				lstrcpy(p, ccui->pi[i].ri[j].throw_list[t]);
				p += lstrlen(ccui->pi[i].ri[j].throw_list[t]);
			}
			if (j == ccui->current_round - 1 &&
				(i > ccui->current_player || (ccui->game_flag == TRUE && i == ccui->current_player))) {
				*(p++) = TEXT(',');
				*(p++) = TEXT(',');
				continue;
			}
			// Mark
			wsprintf(buf, TEXT(",%d"), ccui->pi[i].ri[j].mark_list);
			lstrcpy(p, buf);
			p += lstrlen(buf);
			// Score
			wsprintf(buf, TEXT(",%d"), ccui->pi[i].ri[j].score_list);
			lstrcpy(p, buf);
			p += lstrlen(buf);
		}
		*(p++) = TEXT('\r');
		*(p++) = TEXT('\n');
	}
	*p = TEXT('\0');
	return TRUE;
}

/*
 * score_save - XRA̕ۑ
 */
static BOOL score_save(const HWND hWnd, const CRICKET_COUNTUP_INFO *ccui)
{
	TCHAR path[MAX_PATH];
	TCHAR *buf;
	TCHAR err_str[BUF_SIZE];
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	DWORD size;

	// t@C̐
	if (GetDateFormat(0, 0, NULL, TEXT("yyyyMMdd"), date, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	if (GetTimeFormat(0, 0, NULL, TEXT("HHmmss"), time, BUF_SIZE - 1) == 0) {
		return FALSE;
	}
	wsprintf(path, TEXT("%s_%s_%s.csv"), message_get_res(IDS_STRING_CRICKET_COUNTUP), date, time);

	// t@C̑I
	if (get_save_path(hWnd, message_get_res(IDS_STRING_SAVE_TITLE),
		TEXT("CSV (*.csv)\0*.csv\0All files (*.*)\0*.*\0\0"), TEXT("csv"), path) == FALSE) {
		return FALSE;
	}

	// XRA̍쐬
	size = get_score_string_length(ccui);
	buf = mem_alloc(sizeof(TCHAR) * (size + 1));
	if (buf == NULL) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	get_score_string(ccui, buf);

	if (file_write_buf(path, (BYTE *)buf, sizeof(TCHAR) * size, err_str) == FALSE) {
		mem_free(&buf);
		MessageBox(hWnd, err_str, ERROR_TITLE, MB_ICONERROR);
		return FALSE;
	}
	mem_free(&buf);
	return TRUE;
}

/*
 * get_score - XRA擾
 */
static int get_score(CRICKET_COUNTUP_INFO *ccui, const TCHAR *throw_str, CRICKET_COUNTUP_ROUND_INFO *ri)
{
	int ret = 0;
	int cnt;
	int i;

	// ԍƃJEg擾
	if (*throw_str == TEXT('\0')) {
		return 0;
	} if (lstrcmp(throw_str, message_get_res(IDS_STRING_D_BULL)) == 0) {
		i = 25;
		cnt = 2;
	} else if (lstrcmp(throw_str, message_get_res(IDS_STRING_S_BULL)) == 0) {
		i = 25;
		cnt = 1;
	} else {
		switch (*throw_str) {
		case CHAR_TRIPLE:
			cnt = 3;
			break;
		case CHAR_DOUBLE:
			cnt = 2;
			break;
		case CHAR_SINGLE:
			cnt = 1;
			break;
		}
		i = _ttoi(throw_str + 1);
		if (i == 0) {
			return 0;
		}
	}
	if (ccui->number[ccui->current_round - 1] == i ||
		(ccui->number[ccui->current_round - 1] == 0 && ((i >= 15 && i <= 20) || i == 25))) {
		ret = (i * cnt);
		if (ri != NULL) {
			ri->mark_list += cnt;
			ri->mark[i] += cnt;
		}
	}
	return ret;
}

/*
 * round_draw - Eh̕`
 */
static BOOL round_draw(const BUFFER *draw_bf, RECT *draw_rect, const CRICKET_COUNTUP_INFO *ccui)
{
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	int i;

	hfont = font_create(FONT_NAME, 16, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	i = (ccui->current_round > ccui->round_cnt) ? ccui->round_cnt : ccui->current_round;
	wsprintf(buf, message_get_res(IDS_STRING_DRAW_ROUND), i, ccui->round_cnt);
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), draw_rect, 0);
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * score_draw - XRA̕`
 */
static BOOL score_draw(const BUFFER *draw_bf, const RECT *rect, const CRICKET_COUNTUP_INFO *ccui, const int score)
{
	TEXTMETRIC tm;
	RECT draw_rect;
	HFONT hfont, hRetFont;
	TCHAR buf[BUF_SIZE];
	TCHAR tmp[BUF_SIZE];

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	draw_rect.left += 10;
	draw_rect.right -= 10;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, ccui);

	// vC[
	draw_rect.top += 40;
	hfont = font_create(FONT_NAME, 20, char_set, 0, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	DrawText(draw_bf->draw_dc, ccui->pi[ccui->current_player].name, lstrlen(ccui->pi[ccui->current_player].name), &draw_rect, DT_CENTER);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight + 20;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// ԍ
	hfont = font_create(FONT_NAME, 30, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	lstrcpy(tmp, message_get_res(IDS_STRING_SHOOT_AT));
	switch (ccui->number[ccui->current_round - 1]) {
	case 0:
		wsprintf(buf, TEXT("%s %s"), tmp, message_get_res(IDS_STRING_SHOOT_ALL));
		break;
	case 25:
		wsprintf(buf, TEXT("%s %s"), tmp, message_get_res(IDS_STRING_BULL));
		break;
	default:
		wsprintf(buf, TEXT("%s %d"), tmp, ccui->number[ccui->current_round - 1]);
		break;
	}
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top += tm.tmHeight;
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// XRA
	hfont = font_create(SCORE_FONT_NAME, 100, 0, FW_HEAVY, FALSE, FALSE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	_itot(score, buf, 10);
	DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);
	return TRUE;
}

/*
 * result_draw - ʂ̕`
 */
static BOOL result_draw(const BUFFER *draw_bf, const RECT *rect, const CRICKET_COUNTUP_INFO *ccui)
{
	TEXTMETRIC tm;
	RECT draw_rect, mark_rect;
	HFONT hfont, hRetFont;
	HPEN pen, ret_pen;
	TCHAR buf[BUF_SIZE];
	int score_width;
	int cnt;
	int i, j;
	int n;

	draw_rect = *rect;
	FillRect(draw_bf->draw_dc, &draw_rect, draw_bf->draw_brush);
	draw_rect.left += 10;
	draw_rect.right -= 10;
	mark_rect = draw_rect;
	score_width = draw_rect.right - draw_rect.left;

	SetTextColor(draw_bf->draw_dc, ci.text);
	SetBkColor(draw_bf->draw_dc, ci.background);

	// Eh
	round_draw(draw_bf, &draw_rect, ccui);

	// io[
	hfont = font_create(FONT_NAME, 20, 0, 0, FALSE, TRUE);
	hRetFont = SelectObject(draw_bf->draw_dc, hfont);
	GetTextMetrics(draw_bf->draw_dc, &tm);
	draw_rect.top = 30;
	for (i = 0; i < NUMBER_CNT; i++) {
		j = 0;
		if (ccui->number[i] == 25) {
			lstrcpy(buf, message_get_res(IDS_STRING_BULL));
		} else {
			wsprintf(buf, TEXT("%d"), ccui->number[i]);
		}
		SetTextColor(draw_bf->draw_dc, ci.text);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		// }[N
		for (j = 0; j < ccui->player_cnt; j++) {
			mark_rect.top = draw_rect.top;
			switch (j) {
			case 0: mark_rect.left = 60; break;
			case 1: mark_rect.left = 145; break;
			case 2: mark_rect.left = score_width - 145 - 32; break;
			case 3: mark_rect.left = score_width - 60 - 32; break;
			}
			mark_rect.right = mark_rect.left + 50;
			cnt = 0;
			for (n = 0; n < CRICKET_COUNTUP_ROUND_MAX; n++) {
				cnt += ccui->pi[j].ri[n].mark[ccui->number[i]];
			}
			wsprintf(buf, TEXT("%d"), cnt);
			SetTextColor(draw_bf->draw_dc, ci.focus_background);
			DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &mark_rect, DT_CENTER);
		}
		draw_rect.top += tm.tmHeight + 5;
	}
	SelectObject(draw_bf->draw_dc, hRetFont);
	DeleteObject(hfont);

	// ̕`
	pen = CreatePen(PS_SOLID, 0, ci.line);
	ret_pen = SelectObject(draw_bf->draw_dc, pen);
	MoveToEx(draw_bf->draw_dc, rect->left + 15, draw_rect.top + 5, NULL);
	LineTo(draw_bf->draw_dc, rect->right - 15, draw_rect.top + 5);
	SelectObject(draw_bf->draw_dc, ret_pen);
	DeleteObject(pen);

	// vC[
	draw_rect.top += 10;
	SetTextColor(draw_bf->draw_dc, ci.text);
	for (j = 0; j < ccui->player_cnt; j++) {
		draw_rect.left = j * (score_width / 4) + 22;
		draw_rect.right = draw_rect.left + 100;

		// Average
		hfont = font_create(FONT_NAME, 20, 0, 0, FALSE, TRUE);
		hRetFont = SelectObject(draw_bf->draw_dc, hfont);
		cnt = 0;
		for (n = 0; n < CRICKET_COUNTUP_ROUND_MAX; n++) {
			cnt += ccui->pi[j].ri[n].mark_list;
		}
		_stprintf(buf, TEXT("%.2f"), (float)cnt / (float)CRICKET_COUNTUP_ROUND_MAX);
		DrawText(draw_bf->draw_dc, buf, lstrlen(buf), &draw_rect, DT_CENTER);
		SelectObject(draw_bf->draw_dc, hRetFont);
		DeleteObject(hfont);
	}
	return TRUE;
}

/*
 * score_cricket_countup_proc - EBhEvV[W
 */
static LRESULT CALLBACK score_cricket_countup_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	RECT client_rect, input_rect;
	TCHAR buf[BUF_SIZE];
	int i, j;

	switch (msg) {
	case WM_CREATE:
		// ̓EBhȄ
		input_wnd = score_input_create(hInst, hWnd);
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.right = input_rect.right;
		client_rect.bottom += input_rect.bottom;
		SetWindowPos(hWnd, 0, 0, 0, client_rect.right, client_rect.bottom, SWP_NOMOVE | SWP_NOZORDER);
		SetWindowPos(input_wnd, 0, 0, client_rect.bottom - input_rect.bottom, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER);

		// `p
		hdc = GetDC(hWnd);
		client_rect.bottom -= input_rect.bottom;
		bf.draw_dc = CreateCompatibleDC(hdc);
		bf.draw_bmp = CreateCompatibleBitmap(hdc, client_rect.right, client_rect.bottom);
		bf.draw_ret_bmp = SelectObject(bf.draw_dc, bf.draw_bmp);
		ReleaseDC(hWnd, hdc);
		bf.draw_brush = CreateSolidBrush(ci.background);
		FillRect(bf.draw_dc, &client_rect, bf.draw_brush);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// EBhE̔j
		SendMessage(input_wnd, WM_CLOSE, 0, 0);

		SelectObject(bf.draw_dc, bf.draw_ret_bmp);
		DeleteObject(bf.draw_bmp);
		DeleteDC(bf.draw_dc);
		DeleteObject(bf.draw_brush);
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		SetFocus(input_wnd);
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf.draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_LBUTTONDBLCLK:
		break;

	case WM_GAME_OPTION:
		// countupݒ
		return set_option_cricket_countup(hInst, hWnd, &g_ccui);

	case WM_GAME_INIT:
		// Q[̏
		g_ccui.game_flag = TRUE;

		g_ccui.number[0] = 20;
		g_ccui.number[1] = 19;
		g_ccui.number[2] = 18;
		g_ccui.number[3] = 17;
		g_ccui.number[4] = 16;
		g_ccui.number[5] = 15;
		g_ccui.number[6] = 25;
		g_ccui.number[7] = 0;

		for (i = 0; i < g_ccui.player_cnt; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)g_ccui.pi[i].name);
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE, i, g_ccui.pi[i].score);
		}
		for (; i < PLAYER_MAX; i++) {
			SendMessage(input_wnd, WM_SET_PLAYER_NAME, i, (LPARAM)TEXT(""));
			SendMessage(input_wnd, WM_SET_PLAYER_SCORE_STR, i, (LPARAM)TEXT(""));
		}
		g_ccui.current_round = 1;
		g_ccui.current_player = 0;
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_ccui.pi[g_ccui.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(GetParent(hWnd), WM_SET_BULL_TYPE, g_ccui.op_bull, 0);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_ccui.current_player);
		SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, -1);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		SendMessage(hWnd, WM_DRAW_SCORE, g_ccui.pi[g_ccui.current_player].score, 0);
		break;

	case WM_GAME_STOP:
		break;

	case WM_CHANGE_FOCUS:
		if (g_ccui.game_flag == FALSE) {
			break;
		}
		// XRA擾
		SendMessage(input_wnd, WM_GET_SCORE_STR, 1, (LPARAM)buf);
		i = get_score(&g_ccui, buf, NULL);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 2, (LPARAM)buf);
		i += get_score(&g_ccui, buf, NULL);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 3, (LPARAM)buf);
		i += get_score(&g_ccui, buf, NULL);

		SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_ccui.current_player, g_ccui.pi[g_ccui.current_player].score + i);
		SendMessage(hWnd, WM_DRAW_SCORE, g_ccui.pi[g_ccui.current_player].score + i, 0);
		break;

	case WM_PREV_PLAYER:
		if (g_ccui.current_player <= 0 && g_ccui.current_round <= 1) {
			break;
		}
		SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_ccui.current_player, g_ccui.pi[g_ccui.current_player].score);

		g_ccui.current_player--;
		if (g_ccui.current_player < 0) {
			g_ccui.current_player = g_ccui.player_cnt - 1;
			g_ccui.current_round--;
		}
		if (g_ccui.current_round <= 1) {
			g_ccui.pi[g_ccui.current_player].score = 0;
		} else {
			g_ccui.pi[g_ccui.current_player].score = g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1 - 1].score_list;
		}
		g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1].mark_list = 0;
		ZeroMemory(g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1].mark, sizeof(int) * 26);

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_ccui.pi[g_ccui.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_ccui.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_NEXT_PLAYER:
		// XRA擾
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_GET_SCORE_FULL_STR, i + 1, (LPARAM)g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1].throw_list[i]);
		}
		// XRAݒ
		SendMessage(input_wnd, WM_GET_SCORE_STR, 1, (LPARAM)buf);
		i = get_score(&g_ccui, buf, &g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1]);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 2, (LPARAM)buf);
		i += get_score(&g_ccui, buf, &g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1]);
		SendMessage(input_wnd, WM_GET_SCORE_STR, 3, (LPARAM)buf);
		i += get_score(&g_ccui, buf, &g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1]);
		g_ccui.pi[g_ccui.current_player].score += i;
		g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1].score_list = g_ccui.pi[g_ccui.current_player].score;
		SendMessage(input_wnd, WM_SET_PLAYER_SCORE, g_ccui.current_player, g_ccui.pi[g_ccui.current_player].score);

		// vC[`FW
		g_ccui.current_player++;
		if (g_ccui.current_player >= g_ccui.player_cnt) {
			g_ccui.current_player = 0;
			g_ccui.current_round++;
		}
		if (g_ccui.current_round > g_ccui.round_cnt) {
			// SEhI
			g_ccui.game_flag = FALSE;
			SendMessage(GetParent(hWnd), WM_GAME_END, 0, 0);
			for (j = 0, i = 1; i < g_ccui.player_cnt; i++) {
				if (g_ccui.pi[j].score < g_ccui.pi[i].score) {
					j = i;
				}
			}
			for (i = 0; i < g_ccui.player_cnt; i++) {
				if (j != i && g_ccui.pi[j].score == g_ccui.pi[i].score) {
					break;
				}
			}
			if (g_ccui.player_cnt > 1 && g_ccui.player_cnt > i) {
				// 
				lstrcpy(buf, message_get_res(IDS_STRING_TIE));
			} else {
				SendMessage(input_wnd, WM_SET_WINNER_PLAYER, 0, j);
				wsprintf(buf, message_get_res(IDS_STRING_PLAYER_WIN), g_ccui.pi[j].name);
			}
			SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
			MessageBox(hWnd, buf, message_get_res(IDS_STRING_TITLE_CRICKET_COUNTUP), MB_ICONINFORMATION);
			SendMessage(hWnd, WM_DRAW_SCORE, 0, 0);
			break;
		}

		// 
		wsprintf(buf, message_get_res(IDS_STRING_PLAYER_THROW), g_ccui.pi[g_ccui.current_player].name);
		SendMessage(GetParent(hWnd), WM_SET_STATUS, 0, (LPARAM)buf);
		SendMessage(input_wnd, WM_SET_CURRENT_PLAYER, 0, g_ccui.current_player);
		SendMessage(input_wnd, WM_INIT_SCORE, 0, 0);
		for (i = 0; i < THROW_MAX; i++) {
			SendMessage(input_wnd, WM_SET_SCORE, i + 1, (LPARAM)g_ccui.pi[g_ccui.current_player].ri[g_ccui.current_round - 1].throw_list[i]);
		}
		SendMessage(hWnd, WM_CHANGE_FOCUS, 0, 0);
		break;

	case WM_DRAW_SCORE:
		GetClientRect(hWnd, &client_rect);
		GetClientRect(input_wnd, &input_rect);
		client_rect.bottom -= input_rect.bottom;
		if (g_ccui.game_flag == FALSE) {
			result_draw(&bf, &client_rect, &g_ccui);
		} else {
			score_draw(&bf, &client_rect, &g_ccui, wParam);
		}
		InvalidateRect(hWnd, &client_rect, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SHOW_SCORE:
		if (wParam != -1) {
			g_ccui.list_index = wParam;
		} else {
			g_ccui.list_index = (g_ccui.game_flag == FALSE) ? 0 : g_ccui.current_player;
		}
		show_score_cricket_countup_list(hInst, hWnd, &g_ccui);
		break;

	case WM_SAVE_SCORE:
		score_save(hWnd, &g_ccui);
		break;

	case WM_SET_SCORE:
	case WM_SET_INPUT_ENABLE:
		return SendMessage(input_wnd, msg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_ACCEL_TAB:
		case ID_ACCEL_STAB:
		case ID_ACCEL_RETURN:
		case IDC_BUTTON_NEXT:
			SendMessage(input_wnd, WM_COMMAND, wParam, lParam);
			break;
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_cricket_countup_regist - EBhENX̓o^
 */
BOOL score_cricket_countup_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = (WNDPROC)score_cricket_countup_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_cricket_countup_create - EBhE̍쐬
 */
HWND score_cricket_countup_create(const HINSTANCE hInstance, const HWND pWnd, int id, const int height)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_CHILD | WS_TABSTOP,
		0, 0, SCORE_WIDTH, height, pWnd, (HMENU)id, hInstance, NULL);
	return hWnd;
}
/* End of source */
